/* ------------------------------------------------------------------------------------------------ */
/* FILE :        Abassi.h																			*/
/*																									*/
/* CONTENTS :																						*/
/*               All typedef, definitions and function prototypes for the Abassi RTOS				*/
/*																									*/
/*																									*/
/* Copyright (c) 2011-2013, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*  $Revision: 1.208 $																				*/
/*	$Date: 2013/06/11 19:44:29 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef __ABASSI_H__							/* To protect against re-include					*/
#define __ABASSI_H__

#ifdef __cplusplus
  extern "C" {
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#if !(defined(__KEIL__) && defined(__C51__)) && !defined(__C251__)
  #include <stdint.h> 							/* Keil C51 does not have stdint.h. What is needed	*/
#endif											/* from stdint.h will be patched later on			*/

#ifdef OS_TEST_HOOK								/* File use by Code Time Technologies for testing	*/
  #include "AbassiTestCfg.h"					/* All test RTOS build configurations are there		*/
#endif

#ifdef OS_DEMO									/* File use by Code Time Technologies for the demos	*/
  #include "AbassiDemo.h"						/* All demo RTOS build configurations are there		*/
#endif
												/* Define these in the makefile of set the values	*/
#ifndef OS_DEF_IN_MAKE							/* If the following build options are used			*/
	#define OS_ALLOC_SIZE		0				/* When !=0, RTOS supplied OSalloc					*/
	#define OS_COOPERATIVE		0				/* When !=0, the  kernel is in cooperative mode		*/
	#define OS_EVENTS			0				/* != 0 when event flags are supported				*/
	#define OS_FCFS				0				/* Allow the use of 1st come 1st serve services		*/
	#define OS_IDLE_STACK		0				/* If IdleTask supplied & if so, stack size			*/
	#define OS_LOGGING_TYPE		0				/* Type of logging to use							*/
	#define OS_MAILBOX			0				/* != 0 when mailboxes are used						*/
	#define OS_MAX_PEND_RQST	0U				/* Maximum number of requests performed in ISRs		*/
	#define OS_MEM_BLOCK		0				/* If the block memory pool part of the build		*/
	#define OS_MIN_STACK_USE	0				/* If the kernel minimizes its stack usage			*/
	#define OS_MTX_DEADLOCK		0				/* != 0 to enable mutex deadlock protection			*/
	#define OS_MTX_INVERSION	0				/* >0 priority inheritance, <0 priority ceiling		*/
	#define OS_NAMES			0				/* != 0 when named Tasks / Semaphores / Mailboxes	*/
	#define OS_NESTED_INTS		0				/* != 0 operating with nested interrupts			*/
	#define OS_OUT_OF_MEM		0				/* If trapping out of memory conditions				*/
	#define OS_PRIO_CHANGE		0				/* If a task priority can be changed at run time	*/
	#define OS_PRIO_MIN			0				/* Max priority, Idle = OS_PRIO_MIN, AdameEve = 0	*/
	#define OS_PRIO_SAME		0				/* Support multiple tasks with the same priority	*/
	#define OS_ROUND_ROBIN		0				/* Use round-robin, value specifies period in uS	*/
	#define OS_RUNTIME			0				/* If create Task / Semaphore / Mailbox at run time	*/
	#define OS_SEARCH_ALGO		0				/* If using a fast search							*/
	#define OS_STACK_CHECK		0				/* Set to != for checking stack coillisions			*/
	#define OS_STARVE_PRIO		0				/* Priority threshold for starving protection		*/
	#define OS_STARVE_RUN_MAX	0				/* Maximum Timer Tick for starving protection		*/
	#define OS_STARVE_WAIT_MAX	0				/* Maximum time on hold for starving protection		*/
	#define OS_STATIC_BUF_MBLK	0				/* When OS_STATIC_MBLK != 0, # of memory bytes		*/
	#define OS_STATIC_MBLK		0				/* If !=0 how many block memory descriptors			*/
	#define OS_STATIC_BUF_MBX	0				/* When OS_STATIC_MBX != 0, # of buffer elements	*/
	#define OS_STATIC_MBX		0				/* If !=0 how many mailboxes						*/
	#define OS_STATIC_NAME		0				/* If named mailboxes/semaphore/task, size in char	*/
	#define OS_STATIC_SEM		0				/* If !=0 how many semaphores and mutexes			*/
	#define OS_STATIC_STACK		0				/* if !=0 number of bytes for all stacks			*/
	#define OS_STATIC_TASK		0				/* If !=0 how many tasks (excluding A&E and Idle)	*/
	#define OS_STATIC_TIM_SRV	0				/* If !=0 how many timer services					*/
	#define OS_TASK_SUSPEND		0				/* If a task can suspend another one				*/
	#define OS_TIMEOUT			0				/* !=0 enables blocking timeout						*/
	#define OS_TIMER_CB			0				/* !=0 gives the timer callback period				*/
	#define OS_TIMER_SRV		0				/* !=0 includes the timer services					*/
	#define OS_TIMER_US			0				/* !=0 enables timer & specifies the period in uS	*/
	#define OS_USE_TASK_ARG		0				/* If tasks have arguments							*/
#endif

#ifndef OSputchar
  #define OSputchar(x)	(putchar(x))			/* Character output for ASCII logging				*/
#endif

/* ------------------------------------------------------------------------------------------------	*/
/* ------------------------------------------------------------------------------------------------	*/
/* Compiler & Target processor specifics											   				*/
/* Many defines could be set to default to the most common value but MISRA does not accept #undef	*/
/* So we use an overload definition (OX_nnn) instead of the original (OS_nnn)						*/

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX A9 / CCS																				*/
/* ------------------------------------------------------------------------------------------------ */

#if defined(__TI_TMS470_V4__)																		\
 || defined(__TI_TMS470_V5__)																		\
 || defined(__TI_TMS470_V6__)																		\
 || defined(__TI_TMS470_V7A8__)																		\
 || defined(__TI_TMS470_V7R4__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
   #if (__TI_EABI_ASSEMBLER)
	#define OX_BSS_ZERO				1			/* This ABI zeroes the BSS at start-up				*/
   #else
	#define OX_BSS_ZERO				0			/* TIEABI & TI ARM9 ABI don't zero BSS at start-up	*/
   #endif
	#define OX_MEMSET_LIB			1			/* The function call is smaller than in-line		*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		256			/* Set 256 as it's a very likely maximum			*/
	#endif
	#define OX_NESTED_INTS			0			/* The ARM9 nests FIQ over IRQ: RTOS is only IRQ	*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			0			/* strcpy() under some optimization only works with	*/
												/* strings that are 4 byte aligned. So use in-line	*/
	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/
												/* Functions unique to CCS / A9						*/
	extern volatile unsigned int *COREgetPeriph(void);
	extern void                   GICenable(int IntNmb, int Prio, int Edge);
	extern void                   GICinit(void);
	extern void                   PFabort_Handler(void);
	extern void                   DATAabort_Handler(void);
	extern void                   FIQ_Handler(void);
	extern void                   SWI_Handler(int SWInmb);

	#ifdef __ABASSI_C__							/* Install the protection for multi-threading on	*/
	  #include <_lock.h>						/* the libraries									*/
	  void OS_ARMlock(void);
	  void OS_ARMunlock(void);
	#endif

	#ifdef OS_CMSIS_RTOS						/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__
		int G_OSstarted;
	  #else
		extern int G_OSstarted;
	  #endif
	  #define OX_CMSIS_INIT() 						\
			do {									\
				G_OSstarted = 1;					\
				TSKsetPrio(TSKmyID(), 3);			\
			} while(0)

	#else
	  #define OX_CMSIS_INIT()						\
			do {int _=0;_=_;} while(0)
	#endif

	#define OX_DEVICE_PREINIT()						\
			do {									\
				_register_lock(&OS_ARMlock);		\
				_register_unlock(&OS_ARMunlock);	\
			} while(0)

	#define OX_DEVICE_INIT()						\
		do {										\
			GICinit();								\
			OX_CMSIS_INIT();						\
		} while(0)

	#define OX_XTRA_FCT								\
	void OS_ARMlock(void) {							\
		if (G_OSstate[2] == 0U) {					\
			MTXlock(G_OSmutex, -1);					\
		}											\
		return;										\
	}												\
	void OS_ARMunlock(void) {						\
		if (G_OSstate[2] == 0U) {					\
			MTXunlock(G_OSmutex);					\
		}											\
		return;										\
	}

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M3 / CCS																				*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__TI_TMS470_V7M3__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
   #if (__TI_EABI_ASSEMBLER)
	#define OX_BSS_ZERO				1			/* This ABI zeroes the BSS at start-up				*/
   #else
	#define OX_BSS_ZERO				0			/* TI ARM9 ABI doesn't zero BSS at start-up			*/
   #endif										/* TIABI does not support the Cortex M3				*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		241			/* NVIC has 256 sources of interrupt, drop 15 1st	*/
	#endif
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NVIC) interrupt levels		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is smaller than in-line		*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/
												/* Functions unique to CCS / M3						*/
	extern void BusFault_Handler   (void);
	extern void DebugMon_Handler   (void);
	extern void HardFault_Handler  (void);
	extern void MemManage_Handler  (void);
	extern void NMI_Handler        (void);
	extern void NVIC_enableIRQ     (int IntNmb);
	extern void SVC_Handler        (void);
	extern void UsageFault_Handler (void);

	#ifdef __ABASSI_C__							/* Install the protection for multi-threading on	*/
	  #include <_lock.h>						/* the libraries									*/
	  void OS_ARMlock(void);
	  void OS_ARMunlock(void);
	#endif

	#ifdef OS_CMSIS_RTOS						/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__
		int G_OSstarted;
	  #else
		extern int G_OSstarted;
	  #endif
	  #define OX_CMSIS_INIT() 					\
			do {								\
				G_OSstarted=1;					\
				TSKsetPrio(G_OStaskNow, 3);		\
			} while(0)
	#else
	  #define OX_CMSIS_INIT()					\
			do {int _=0;_=_;} while(0)
	#endif

	#define OX_DEVICE_PREINIT()					\
		do {									\
			_register_lock(&OS_ARMlock);		\
			_register_unlock(&OS_ARMunlock);	\
		} while(0)

	#define OX_DEVICE_INIT()					\
		do {									\
			OX_CMSIS_INIT();					\
		} while(0)

	#define OX_XTRA_FCT							\
	void OS_ARMlock(void) {						\
		if (G_OSstate[2] == 0U) {				\
			MTXlock(G_OSmutex, -1);				\
		}										\
		return;									\
	}											\
	void OS_ARMunlock(void) {					\
		if (G_OSstate[2] == 0U) {				\
			MTXunlock(G_OSmutex);				\
		}										\
		return;									\
	}

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / Atollic / GCC															*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__)																				\
   && (defined(__ARM_ARCH_6M__) || defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__))
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the Cortex M3				*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#ifndef OS_N_INTERRUPTS
	 #ifdef __ARM_ARCH_6M__						/* *** CORTEX M0									*/
	  #define OX_N_INTERRUPTS		49			/* Cortex M0 has 64 interrupts max, drop 15 1st		*/
	 #else										/* *** CORTEX M3 / M4								*/
	  #define OX_N_INTERRUPTS		241			/* Cortex M3/M4 have 512 interrupts max, use 1/2	*/
	 #endif
	#endif
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NVIC) interrupt levels		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	extern void BusFault_Handler   (void);
	extern void DebugMon_Handler   (void);
	extern void HardFault_Handler  (void);
	extern void MemManage_Handler  (void);
	extern void NMI_Handler        (void);
	extern void SVC_Handler        (void);
	extern void UsageFault_Handler (void);

	#ifdef OS_CMSIS_RTOS						/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__
		int G_OSstarted;
	  #else
		extern int G_OSstarted;
	  #endif
	  #define OX_CMSIS_INIT() 					\
			do {								\
				G_OSstarted=1;					\
				TSKsetPrio(G_OStaskNow, 3);		\
			} while(0)
	#else
	  #define OX_CMSIS_INIT()					\
			do {int _=0;_=_;} while(0)
	#endif

	#ifdef OS_ATOLLIC_REENT						/* For Atollic re-entrance, the RTOS deals 2 ways	*/
	  #if ((OS_ATOLLIC_REENT) != 0)				/* When +ve, the RTOS allows standard functions		*/
												/* by updating _impure_ptr.  When -ve, holds _reent	*/
	  	#include <reent.h>						/* but user must use the _xxxx_r() function			*/

	    #define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		#if ((OS_ATOLLIC_REENT) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
						struct _reent *_Reent;													\
						_Reent = (struct _reent *)OSalloc(sizeof(struct _reent));				\
						 (_TskDsc)->XtraData[0] = (intptr_t)_Reent;								\
						_REENT_INIT_PTR(_Reent);												\
					}																			\
					else {						/* A&E uses the default data		 		*/	\
						(_TskDsc)->XtraData[0] = (intptr_t)_impure_ptr;							\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
					struct _reent *_Reent;														\
					(_TskDsc)->XtraData[0] = (intptr_t)&(thread_def->Reent);					\
					_Reent = (struct _reent *)&(thread_def->Reent);								\
					_REENT_INIT_PTR(_Reent);													\
				} while(0)
			#endif
		#endif

		#ifdef __ABASSI_C__
			#include <envlock.h>
			#include <locale.h>
			#include <malloc.h>
			int G_OSmtxOK;
		  #if ((OS_ATOLLIC_REENT) < 0)
			struct _reent *G_OSdefLibCtx;
		  #endif
		#endif

		#if ((OS_ATOLLIC_REENT) < 0)
			#define OX_DEVICE_PREINIT()						\
				do {										\
					G_OSdefLibCtx = _impure_ptr;			\
				} while(0)
		#endif

		#define OX_DEVICE_INIT()							\
			do {											\
				OX_CMSIS_INIT();							\
				G_OSmtxOK = 1;								\
			} while(0)

		#define OX_XTRA_FCT									\
			void __malloc_lock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __malloc_unlock(struct _reent *reent) {	\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __env_lock(struct _reent *reent) {			\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __env_unlock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __tz_lock(void) {							\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __tz_unlock(void) {						\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}
	  #endif
	#endif

	#ifndef OX_DEVICE_INIT
	  #define OX_DEVICE_INIT()								\
		do {												\
			OX_CMSIS_INIT();								\
		} while(0)
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / IAR																	*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCARM__) && defined(__ARM_PROFILE_M__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* IAR zeroes the BSS for the Cortex M#				*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#ifndef OS_N_INTERRUPTS
	 #if __CORE__ == __ARM6M__					/* *** CORTEX M0									*/
	  #define OX_N_INTERRUPTS		49			/* Cortex M0 has 64 interrupts max, drop 15 1st		*/
	 #else										/* *** CORTEX M3 / M4								*/
	  #define OX_N_INTERRUPTS		241			/* Cortex M3/M4 have 512 interrupts max, use 1/2	*/
	 #endif
	#endif
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NVIC) interrupt levels		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	extern void BusFault_Handler   (void);
	extern void DebugMon_Handler   (void);
	extern void HardFault_Handler  (void);
	extern void MemManage_Handler  (void);
	extern void NMI_Handler        (void);
	extern void SVC_Handler        (void);
	extern void UsageFault_Handler (void);

	#ifdef OS_CMSIS_RTOS						/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__
		int G_OSstarted;
	  #else
		extern int G_OSstarted;
	  #endif
	  #define OX_CMSIS_INIT() 					\
			do {								\
				G_OSstarted=1;					\
				TSKsetPrio(G_OStaskNow, 3);		\
			} while(0)
	#else
	  #define OX_CMSIS_INIT()					\
			do {int _=0;_=_;} while(0)
	#endif

	#ifdef __ABASSI_C__							/* G_OSmtxOK is needed for multi-thread protection	*/
		int G_OSmtxOK;							/* No need to set to zero as BSS is zeroed			*/
	#endif

	#if defined(OS_IAR_MTHREAD)
  	  #if ((OS_IAR_MTHREAD) != 0)				/* Library multi-threading protection				*/

		#include <yvals.h>

		#define OX_TASK_XTRA_FIELD	1			/* Needed to hold local multi-thread data pointer	*/

		#if ((OS_IAR_MTHREAD) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
					    void *MTptr;															\
						MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);							\
						__iar_dlib_perthread_initialize(MTptr);									\
						(_TskDsc)->XtraData[0] = (intptr_t)MTptr;								\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
				    void *MTptr;																\
					MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);								\
					__iar_dlib_perthread_initialize(MTptr);										\
					(_TskDsc)->XtraData[0] = (intptr_t)MTptr;									\
				} while(0)
			#endif
		#endif

		#define OX_XTRA_FCT																		\
			void _DLIB_TLS_MEMORY *__iar_dlib_perthread_access(void _DLIB_TLS_MEMORY *SymbPtr)	\
			{																					\
			char _DLIB_TLS_MEMORY *MTptr;														\
				MTptr = (char _DLIB_TLS_MEMORY *)__segment_begin("__DLIB_PERTHREAD");			\
				if (TSKmyID() != (TSK_t *)NULL) {												\
					if (TSKmyID()->XtraData[0] != (intptr_t)0) {								\
					    MTptr = (void *)TSKmyID()->XtraData[0];									\
					}																			\
				}																				\
				MTptr += __IAR_DLIB_PERTHREAD_SYMBOL_OFFSET(SymbPtr);							\
				return((void _DLIB_TLS_MEMORY *)MTptr);											\
			}
	  #endif
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			OX_CMSIS_INIT();					\
			G_OSmtxOK = 1;						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* ARM CORTEX M0 & M3 & M4 / Keil																	*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__CC_ARM)	   						/* Keil RealView ARM Compiler						*/
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* Keil zeroes the BSS for the Cortex M#			*/
	#define OX_ISR_TBL_OFFSET		1			/* We need to include SysTick (interrupt # == -1)	*/
	#define OX_MEMSET_LIB			0			/* The function call is less efficient than in-line	*/
	#ifndef OS_N_INTERRUPTS
	 #ifdef __TARGET_CPU_CORTEX_M0				/* *** CORTEX M0									*/
	  #define OX_N_INTERRUPTS		49			/* Cortex M0 has 64 interrupts max, drop 15 1st		*/
	 #else										/* *** CORTEX M3 / M4								*/
	  #define OX_N_INTERRUPTS		241			/* Cortex M3/M4 have 512 interrupts max, use 1/2	*/
	 #endif
	#endif
	#define OX_NESTED_INTS			1			/* The ARM has 8 nested (NVIC) interrupt levels		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	extern void BusFault_Handler   (void);
	extern void DebugMon_Handler   (void);
	extern void HardFault_Handler  (void);
	extern void MemManage_Handler  (void);
	extern void NMI_Handler        (void);
	extern void SVC_Handler        (void);
	extern void UsageFault_Handler (void);

	#ifdef OS_CMSIS_RTOS						/* Special stuff for CMSIS V3.0						*/
	  #ifdef __ABASSI_C__
		int G_OSstarted;
	  #else
		extern int G_OSstarted;
	  #endif
	  #define OX_CMSIS_INIT() 					\
			do {								\
				G_OSstarted=1;					\
				TSKsetPrio(G_OStaskNow, 3);		\
			} while(0)
	#else
	  #define OX_CMSIS_INIT()					\
			do {int _=0;_=_;} while(0)
	#endif

	#if defined(OS_KEIL_REENT) && !defined(__MICROLIB)
  	  #if ((OS_KEIL_REENT) != 0)				/* Library multi-threading protection				*/

		#define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		extern char __libspace_start[96];		/* Default libspace area							*/

		#ifdef __ABASSI_C__
			int G_OSmtxOK;						/* No need to set to zero as BSS is zeroed			*/
		#endif

		#if ((OS_KEIL_REENT) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
						void *_Reent;															\
						_Reent = OSalloc(96);													\
						 (_TskDsc)->XtraData[0] = (intptr_t)_Reent;								\
						memset(_Reent, 0, 96);													\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
					void *_Reent;																\
					_Reent = OSalloc(96);														\
					 (_TskDsc)->XtraData[0] = (intptr_t)_Reent;									\
					memset(_Reent, 0, 96);														\
				} while(0)
			#endif
		#endif

		#define OX_XTRA_FCT									\
		int _mutex_initialize(SEM_t *Sema) {				\
			Sema = Sema;									\
			return(1);										\
		}													\
		void _mutex_acquire(SEM_t *Sema) {					\
			Sema = Sema;									\
			if (G_OSstate[2] == 0) { 						\
			if (G_OSmtxOK != 0) {							\
				MTXlock(G_OSmutex, -1);						\
			}}												\
		}													\
		void _mutex_release(SEM_t *Sema) {					\
			Sema = Sema;									\
			if (G_OSstate[2] == 0) { 						\
			if (G_OSmtxOK != 0) {							\
				MTXunlock(G_OSmutex);						\
			}}												\
		}													\
		void _mutex_free(SEM_t *Sema) {						\
			Sema = Sema;									\
			return;											\
		}													\
		void *__user_perproc_libspace(void) {				\
			return(&__libspace_start[0]);					\
		}													\
		void *__user_perthread_libspace(void) { 			\
			void *Ptr;										\
			Ptr = (void *)&__libspace_start[0];				\
			if (TSKmyID() != (TSK_t *)NULL) {				\
				if (TSKmyID()->XtraData[0] != (intptr_t)0) {\
				    Ptr = (void *)TSKmyID()->XtraData[0];	\
				}											\
			}												\
			return(Ptr);									\
		}

		#define OX_DEVICE_INIT()							\
			do {											\
				OX_CMSIS_INIT();							\
				G_OSmtxOK = 1;								\
			} while(0)
	  #endif
	#endif

	#ifndef OX_DEVICE_INIT
		#define OX_DEVICE_INIT()							\
			do {											\
				OX_CMSIS_INIT();							\
			} while(0)
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* AVR / GCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__AVR__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the AVR					*/
	#define OX_INT_1_READ			0			/* AVR is 8 bit and ints are 16 bits: 2 reads		*/
	#define OX_ISR_TBL_OFFSET		(-1)		/* Reset is 0, 1 is first available					*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		34			/* ATmega128: 35 but Reset is not included in table	*/
	#endif
	#ifndef OS_NESTED_INTS
	  #define OX_NESTED_INTS		0
	#endif
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* AVR / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCAVR__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* IAR zeroes the BSS for the AVR					*/
	#define OX_INT_1_READ			0			/* AVR is 8 bit and ints are 16 bits: 2 reads		*/
	#define OX_ISR_TBL_OFFSET		(-1)		/* Reset is 0, 1 is first available					*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		34			/* ATmega128: 35 but Reset is not included in table	*/
	#endif
	#ifndef OS_NESTED_INTS
	  #define OX_NESTED_INTS		0
	#endif
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* AVR / IAR support the multi-thread mutex			*/
		int G_OSmtxOK;							/* protection. It needs to knwo if the mutex is OK	*/
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			G_OSmtxOK = 1;						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* AVR32A / AVR32 Studio																			*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__AVR32_AVR32A__)
	#ifndef OS_AVR32A_INT_GRP
	  #define OS_AVR32A_INT_GRP		19			/* Number of interrupt groups for this device		*/
	#endif
	#ifndef OS_AVR32A_INT_LINE
	  #define OS_AVR32A_INT_LINE	10			/* Largest number of interrupt lines in a group		*/
	#endif
	#define OS_AVR32A_INTC_REG		0xFFFF0800	/* Base address of the INTC register bank			*/

	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the AVR32					*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#define OX_N_INTERRUPTS			((OS_AVR32A_INT_GRP)*(OS_AVR32A_INT_LINE))
	#define OX_NESTED_INTS			1			/* The AVR32 has 4 nested interrupt levels			*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			0			/* The function call is less efficient than in-line	*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#define OSavr32aISRmap(_Grp, _Line)		(((_Grp)*OS_AVR32A_INT_LINE)+(_Line))
	#define OSavr32aISRprio(_Grp, _Prio)	(((int *)OS_AVR32A_INTC_REG)[_Grp]=G_OSavrPrio[_Prio])

	extern int G_OSavrPrio[4];

	#ifdef __ABASSI_C__
		extern void OSavrSetEBVA(void);
	#endif

	#ifdef OS_NEWLIB_REENT						/* For NewLib re-entrance, the RTOS deals 2 ways	*/
	  #if ((OS_NEWLIB_REENT) != 0)				/* When +ve, the RTOS allows standard functions		*/
												/* by updating _impure_ptr.  When -ve, holds _reent	*/
		#include <reent.h>						/* but user must use the _xxxx_r() function			*/

	    #define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		#if ((OS_NEWLIB_REENT) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
						struct _reent *_Reent;													\
						_Reent = (struct _reent *)OSalloc(sizeof(struct _reent));				\
						 (_TskDsc)->XtraData[0] = (intptr_t)_Reent;								\
						_REENT_INIT_PTR(_Reent);												\
					}																			\
					else {						/* A&E uses the default data		 		*/	\
						(_TskDsc)->XtraData[0] = (intptr_t)_impure_ptr;							\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
					struct _reent *_Reent;														\
					(_TskDsc)->XtraData[0] = (intptr_t)&(thread_def->Reent);					\
					_Reent = (struct _reent *)&(thread_def->Reent);								\
					_REENT_INIT_PTR(_Reent);													\
				} while(0)
			#endif
		#endif

		#ifdef __ABASSI_C__
			#include <locale.h>
			#include <malloc.h>
			int G_OSmtxOK;
		  #if ((OS_NEWLIB_REENT) < 0)
			struct _reent *G_OSdefLibCtx;
		  #endif
		#endif

		#if ((OS_NEWLIB_REENT) < 0)
			#define OX_DEVICE_PREINIT()						\
				do {										\
					G_OSdefLibCtx = _impure_ptr;			\
				} while(0)
		#endif

		#define OX_DEVICE_INIT()							\
			do {											\
				OSavrSetEBVA();								\
				G_OSmtxOK = 1;								\
			} while(0)

		#define OX_XTRA_FCT									\
			void __malloc_lock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __malloc_unlock(struct _reent *reent) {	\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __env_lock(struct _reent *reent) {			\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __env_unlock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __tz_lock(void) {							\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __tz_unlock(void) {						\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}
	  #endif
	#endif

	#ifndef OX_DEVICE_INIT
		#define OX_DEVICE_INIT()							\
			do {											\
				OSavrSetEBVA();								\
			} while(0)
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* AVR32 / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCAVR32__)
  #if (__CORE__ == __AVR32A__)
	#ifndef OS_AVR32A_INT_GRP
	  #define OS_AVR32A_INT_GRP		19			/* Number of interrupt groups for this device		*/
	#endif
	#ifndef OS_AVR32A_INT_LINE
	  #define OS_AVR32A_INT_LINE	10			/* Largest number of interrupt lines in a group		*/
	#endif
	#define OS_AVR32A_INTC_REG		0xFFFF0800	/* Base address of the INTC register bank			*/

	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* IAR zeroes the BSS for the AVR					*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#define OX_N_INTERRUPTS			((OS_AVR32A_INT_GRP)*(OS_AVR32A_INT_LINE))
	#define OX_NESTED_INTS			1			/* The AVR32 has 4 nested interrupt levels			*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#define OSavr32aISRmap(_Grp, _Line)			(((_Grp)*OS_AVR32A_INT_LINE)+(_Line))

	extern void OSavr32aISRprio(int Grp, int Prio);	/* Can't use #define: bad code from compiler	*/

	#ifdef __ABASSI_C__							/* G_OSmtxOK is needed for multi-thread protection	*/
		int G_OSmtxOK;							/* No need to set to zero as BSS is zeroed			*/
	#endif

	#if defined(OS_IAR_MTHREAD)
  	  #if ((OS_IAR_MTHREAD) != 0)				/* Library multi-threading protection				*/

		#include <yvals.h>

		#define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		#if ((OS_IAR_MTHREAD) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
					    void *MTptr;															\
						MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);							\
						__iar_dlib_perthread_initialize(MTptr);									\
						(_TskDsc)->XtraData[0] = (intptr_t)MTptr;								\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
				    void *MTptr;																\
					MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);								\
					__iar_dlib_perthread_initialize(MTptr);										\
					(_TskDsc)->XtraData[0] = (intptr_t)MTptr;									\
				} while(0)
			#endif
		#endif

		#define OX_XTRA_FCT																		\
			void _DLIB_TLS_MEMORY *__iar_dlib_perthread_access(void _DLIB_TLS_MEMORY *SymbPtr)	\
			{																					\
			char _DLIB_TLS_MEMORY *MTptr;														\
				MTptr = (char _DLIB_TLS_MEMORY *)__segment_begin("__DLIB_PERTHREAD");			\
				if (TSKmyID() != (TSK_t *)NULL) {												\
					if (TSKmyID()->XtraData[0] != (intptr_t)0) {								\
					    MTptr = (void *)TSKmyID()->XtraData[0];									\
					}																			\
				}																				\
				MTptr += __IAR_DLIB_PERTHREAD_SYMBOL_OFFSET(SymbPtr);							\
				return((void _DLIB_TLS_MEMORY *)MTptr);											\
			}
	  #endif
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			G_OSmtxOK = 1;						\
		} while(0)

  #else
	#error "Abassi: Unsupported processor/compiler"

  #endif

/* ------------------------------------------------------------------------------------------------ */
/* C28x / CCS																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif (defined(__TI_COMPILER_VERSION__) && defined(__TMS320C28XX__))

	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				0			/* CCS / C28X does not zero the BSS at start-up		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			-1			/* OSmapISR() is used instead of OSisrInstall()		*/
	#ifndef OS_NESTED_INTS
	  #define OX_NESTED_INTS		0			/* Interrupt nesting is disabled by default			*/
	#endif
	#define OX_STACK_GROWTH			1			/* Stack grows toward higher memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/
												/* RTOS timer used is CPU-Timer #2, re-enable INT14	*/
	#define OX_TIM_TICK_ACK() do{asm(" or IER, #0x2000");}while(0)

	typedef long OSalign_t;						/* Data type that will use the alignment size		*/

	extern void OSmapISR(int VectID, int TblNmb, void (* FctPtr)(void));

	#define OX_XTRA_BSS_ZERO()					\
			G_OSmtxOK = 0;

	#ifdef __ABASSI_C__							/* This code is added when in Abassi.c				*/
	  #include <_lock.h>
	  void OS_C28Xlock(void);
	  void OS_C28Xunlock(void);
	  extern void InitPieCtrl(void);
	  int G_OSmtxOK = 0;
	#endif

	#define OX_DEVICE_PREINIT()					\
			InitPieCtrl();						\
												/* Device specific init: attach mutex to library	*/
	#define OX_DEVICE_INIT()					\
		do {									\
			_register_lock(&OS_C28Xlock);		\
			_register_unlock(&OS_C28Xunlock);	\
			G_OSmtxOK = 1;						\
		} while(0)
												/* Device specific fct: library mutex				*/
	#define OX_XTRA_FCT							\
	void OS_C28Xlock(void) {					\
		if ((G_OSmtxOK != 0)					\
		&&  (G_OSstate[2] == 0)) {				\
			MTXlock(G_OSmutex, -1);				\
		}										\
	}											\
	void OS_C28Xunlock(void) {					\
		if ((G_OSmtxOK != 0)					\
		&&  (G_OSstate[2] == 0)) {				\
			MTXunlock(G_OSmutex);				\
		}										\
	}

/* ------------------------------------------------------------------------------------------------ */
/* COLDFIRE / IAR																					*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICCCF__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* IAR zeroes the BSS for the Coldfire				*/
	#define OX_ISR_TBL_OFFSET		(-64)		/* First ISR is #64									*/
	#define OX_MEMSET_LIB			0			/* The function call same as in-line, keep in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		192			/* CF has total of 256 but first 64 are reserved	*/
	#endif
	#define OX_NESTED_INTS			1			/* CF has 8 nested interrupt levels					*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int32_t OSalign_t;					/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* PIC32 / MPLAB GCC																				*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__PIC32MX__)
	#include <plib.h>
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the PIC32					*/
	#define OX_MEMSET_LIB			0			/* With -Os in-line is more compact					*/
	#define OX_N_INTERRUPTS			64			/* PIC32 has 64 sources of interrupt				*/
	#define OX_NESTED_INTS			1			/* The PIC32 has 8 nested interrupt levels			*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* With -Os in-line is more compact					*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* Set this pragma only once						*/
	  #pragma interrupt ISRdispatch single vector 0	/* Needed to configure the interrupt controller	*/
	#endif										/* in single vector mode							*/

	#define OX_DEVICE_INIT()					\
		do {									\
			OSdint();							\
			INTEnableSystemSingleVectoredInt();	\
			mINTSingleVectorRegistorSelect(0);	\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / CCS																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__TI_COMPILER_VERSION__) && defined(__MSP430__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* EABI zeros, COFF is with _system_pre_init()		*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__
	  #include <msp430.h>
	  #include <_lock.h>
	  extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/
	  void OS_MSPlock(void);					/* Multi-thread mutex protection					*/
	  void OS_MSPunlock(void);					/* Multi-thread mutex protection					*/
	  int  _system_pre_init(void);				/* Start-up add on									*/
	#endif
												/* Device specific init: attach mutex to library	*/
	#define OX_DEVICE_INIT()					\
		do {									\
			InitISRtbl();						\
		     _register_lock(&OS_MSPlock);		\
		     _register_unlock(&OS_MSPunlock);	\
		} while(0)
												/* Device specific fct: BSS zeroing & library mutex	*/

   #ifdef __TI_EABI__							/* EABI library init BSS to zero at start-up		*/
	#define OX_DOING_BSS_0()					\
		do { int _=0;_=_;} while(0)
   #else										/* COFF library doen't init BSS to zero at start-up	*/
	#define OX_DOING_BSS_0()					\
		do {									\
		  extern int  __cinit__;				\
		  extern char __bss__;					\
		  extern char __end__;					\
		  if (&__cinit__ != (int *)-1) {		\
    	    char *cptr;							\
			WDTCTL = WDTPW + WDTHOLD;			\
	        for(cptr=&__bss__;cptr<=&__end__ ;){\
   		        *cptr++ = 0;					\
			}									\
			WDTCTL = WDTPW;						\
		  }										\
		} while(0)
   #endif

	#define OX_XTRA_FCT							\
	int _system_pre_init(void) {				\
		OX_DOING_BSS_0();						\
		_register_lock(&_nop);					\
		_register_unlock(&_nop);				\
		return(1);								\
	}											\
	void OS_MSPlock(void) {						\
		if  (G_OSstate[2] == 0) {				\
			MTXlock(G_OSmutex, -1);				\
		}										\
	}											\
	void OS_MSPunlock(void) {					\
		if (G_OSstate[2] == 0) {				\
			MTXunlock(G_OSmutex);				\
		}										\
	}

/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / GCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__) && defined(__MSP430__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the MSP430				*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__
	  extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/
	#endif

	#define OX_DEVICE_INIT()					\
		do {									\
			InitISRtbl();						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* MSP430 / IAR																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICC430__)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* IAR zeroes the BSS for the MSP430				*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			0			/* MSP430 table is in assembly file (16/32/64)		*/
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef int OSalign_t;						/* Data type that will use the alignment size		*/

	#ifdef __ABASSI_C__							/* Used with the multi-thread mutes protection		*/
	  extern void InitISRtbl(void);				/* ISR table initialization in assembly file		*/
	  int G_OSmtxOK;							/* No need to set to 0 as BSS is zeroed				*/
	#endif

	#if defined(OS_IAR_MTHREAD)
  	  #if ((OS_IAR_MTHREAD) != 0)				/* Library multi-threading protection				*/

		#include <yvals.h>

		#define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		#if ((OS_IAR_MTHREAD) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
					    void *MTptr;															\
						MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);							\
						__iar_dlib_perthread_initialize(MTptr);									\
						(_TskDsc)->XtraData[0] = (intptr_t)MTptr;								\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
				    void *MTptr;																\
					MTptr  = OSalloc(__IAR_DLIB_PERTHREAD_SIZE);								\
					__iar_dlib_perthread_initialize(MTptr);										\
					(_TskDsc)->XtraData[0] = (intptr_t)MTptr;									\
				} while(0)
			#endif
		#endif

		#define OX_XTRA_FCT																		\
			void _DLIB_TLS_MEMORY *__iar_dlib_perthread_access(void _DLIB_TLS_MEMORY *SymbPtr)	\
			{																					\
			char _DLIB_TLS_MEMORY *MTptr;														\
				MTptr = (char _DLIB_TLS_MEMORY *)__segment_begin("__DLIB_PERTHREAD");			\
				if (TSKmyID() != (TSK_t *)NULL) {												\
					if (TSKmyID()->XtraData[0] != (intptr_t)0) {								\
					    MTptr = (void *)TSKmyID()->XtraData[0];									\
					}																			\
				}																				\
				MTptr += __IAR_DLIB_PERTHREAD_SYMBOL_OFFSET(SymbPtr);							\
				return((void _DLIB_TLS_MEMORY *)MTptr);											\
			}
	  #endif
	#endif

	#define OX_DEVICE_PREINIT()					\
			G_OSmtxOK = 1;

	#define OX_DEVICE_INIT()					\
		do {									\
			InitISRtbl();						\
		} while(0)

/* ------------------------------------------------------------------------------------------------ */
/* Tensilica's Xtensa																				*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__GNUC__)																				\
   && (defined(__XTENSA__) || defined(_XCC__))
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				1			/* GCC zeroes the BSS for the Xtensa				*/
	#define OX_ISR_TBL_OFFSET		0			/* No offset in the interrupt table					*/
	#define OX_MEMSET_LIB			0			/* Library call is smaller code size				*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		8
	#endif
	#define OX_NESTED_INTS			0
	#define OX_STACK_GROWTH			(-1)		/* Stack grows toward lower memory addresses		*/
	#define OX_USE_STRLIB			1			/* Same size for lib & in-line: use in-line			*/

	typedef int64_t OSalign_t;					/* Data type that will use the alignment size		*/

	#ifdef OS_NEWLIB_REENT						/* For NewLib re-entrance, the RTOS deals 2 ways	*/
	  #if ((OS_NEWLIB_REENT) != 0)				/* When +ve, the RTOS allows standard functions		*/
												/* by updating _impure_ptr.  When -ve, holds _reent	*/
		#include <reent.h>						/* but user must use the _xxxx_r() function			*/

	    #define OX_TASK_XTRA_FIELD	1			/* Needed to hold the local reentrant data pointer	*/

		#if ((OS_NEWLIB_REENT) > 0)
			#ifdef __ABASSI_C__
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)												\
				do {																			\
					if (_Fct != NULL) {															\
						struct _reent *_Reent;													\
						_Reent = (struct _reent *)OSalloc(sizeof(struct _reent));				\
						 (_TskDsc)->XtraData[0] = (intptr_t)_Reent;								\
						_REENT_INIT_PTR(_Reent);												\
					}																			\
					else {						/* A&E uses the default data		 		*/	\
						(_TskDsc)->XtraData[0] = (intptr_t)_impure_ptr;							\
					}																			\
				} while(0)
			#else								/* When used outside of Abassi.c by TSK_SETUP()	*/
		  	  #define OX_TASK_XTRA(_TskDsc, _Fct)	/* there is no need to check for A&E	*/	\
				do {																			\
					struct _reent *_Reent;														\
					(_TskDsc)->XtraData[0] = (intptr_t)&(thread_def->Reent);					\
					_Reent = (struct _reent *)&(thread_def->Reent);								\
					_REENT_INIT_PTR(_Reent);													\
				} while(0)
			#endif
		#endif

		#ifdef __ABASSI_C__
			#include <locale.h>
			#include <malloc.h>
			int G_OSmtxOK;
		  #if ((OS_NEWLIB_REENT) < 0)
			struct _reent *G_OSdefLibCtx;
		  #endif
		#endif

		#if ((OS_NEWLIB_REENT) < 0)
			#define OX_DEVICE_PREINIT()						\
				do {										\
					G_OSdefLibCtx = _impure_ptr;			\
				} while(0)
		#endif

		#define OX_DEVICE_INIT()							\
			do {											\
				G_OSmtxOK = 1;								\
			} while(0)

		#define OX_XTRA_FCT									\
			void __malloc_lock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __malloc_unlock(struct _reent *reent) {	\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __env_lock(struct _reent *reent) {			\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __env_unlock(struct _reent *reent) {		\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}												\
			void __tz_lock(void) {							\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXlock(G_OSmutex, -1);					\
				}}											\
			}												\
			void __tz_unlock(void) {						\
				if (G_OSstate[2] == 0) { 					\
				if (G_OSmtxOK != 0) {						\
					MTXunlock(G_OSmutex);					\
				}}											\
			}
	  #endif
	#endif

/* ------------------------------------------------------------------------------------------------ */
/* Z80 / SDCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(SDCC) && defined(SDCC_z80)
	#define OX_AE_STACK_SIZE		0			/* A&E stack is main()'s, no user overload			*/
	#define OX_BSS_ZERO				0			/* SDCC does not zero the BSS for the Z80			*/
	#define OX_INT_1_READ			0			/* Reading an int requires 2 read ops				*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			8			/* Z80 has 8 sources of interrupt					*/
	#define OX_NESTED_INTS			0			/* Z80 ints are level, so can't use nested ints		*/
	#define OX_USE_INT_REGISTER		1			/* Keep track of ISR enable/disable with variable	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* 8051 / Keil																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__KEIL__)	&& defined(__C51__)		/* KEIL COMPILER									*/
	#ifndef OS_AE_STACK_SIZE
	  #define OX_AE_STACK_SIZE		512			/* If AE stack not defined, give it a default value	*/
	#endif
	#ifndef OS_BSS_ZERO
	  #define OX_BSS_ZERO			0			/* Keil can zero the BSS, by default is not enable	*/
	#else
	  #define OX_BSS_ZERO			(OS_BSS_ZERO)
	#endif
	#ifndef OS_CPU_FREQ
	  #define OS_CPU_FREQ			11059200	/* Sometimes needed									*/
	#endif
	#define OX_INT_1_READ			0			/* Reading an int requires 2 read ops				*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		7			/* Basic 8051 has 6 interrupts (+1 extra as example)*/
	#endif
	#define OX_NESTED_INTS			1			/* The 8051 uses a 2 priority scheme for interrupts	*/
	#ifndef OS_SERIAL_BAUD
	  #define OS_SERIAL_BAUD		19200		/* Default baudrate for the serial port				*/
	#endif
	#define OX_STACK_GROWTH			0			/* Can't do stack monitoring on 8051, 0 disables it	*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/
	#define REENT_POST				reentrant	/* KEIL C51 needs to be told about re-entrance		*/

	#pragma REGPARMS							/* Use registers to pass arguments					*/
	typedef long intptr_t;						/* Keil: no stdint.h. Ptr are 3 bytes use 4 bytes	*/
	typedef long int32_t;						/* Same here										*/
	typedef unsigned long uint32_t;				/* Same here										*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* 80251 / Keil																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__C251__)							/* KEIL COMPILER									*/
	#if ((__MODEL__) == 2)						/* AE stack only needed for Large data model		*/
	 #ifndef OS_AE_STACK_SIZE
	  #define OX_AE_STACK_SIZE		512			/* If A&E stack not defined, set the default value	*/
	 #else
	  #define OX_AE_STACK_SIZE		(OS_AE_STACK_SIZE)
	 #endif
	#else
	  #define OX_AE_STACK_SIZE		0			/* A&E stack not needed for XTiny & XSmall models	*/
	#endif
	#ifndef OS_BSS_ZERO
	  #define OX_BSS_ZERO			0			/* Keil can zero the BSS, by default is not enable	*/
	#else
	  #define OX_BSS_ZERO			(OS_BSS_ZERO)
	#endif
	#ifndef OS_CPU_FREQ
	  #define OS_CPU_FREQ			11059200	/* Sometimes needed									*/
	#endif
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#ifndef OS_N_INTERRUPTS
	  #define OX_N_INTERRUPTS		8			/* Basic 80251 has 7 interrupts (+1 for the example)*/
	#endif
	#define OX_NESTED_INTS			1			/* The 80251 uses a 4 priority scheme for interrupts*/
	#ifndef OS_SERIAL_BAUD
	  #define OS_SERIAL_BAUD		19200		/* Default baudrate for the serial port				*/
	#endif
	#define OX_STACK_GROWTH			1			/* Stack grows toward higher memory addresses		*/
	#if ((__MODEL__) == 2) || ((__MODEL__) == 5)/* 2:Large / 5:XSmall (warning if not declared far)	*/
	  typedef char far OSstrCnst_t;				/* Initialized const strings are in far space		*/
	#else
	  typedef char OSstrCnst_t;					/* Initialized const strings are in regular space	*/
	#endif
	#define OX_STR_CONST			1			/* Tag the typedef was set							*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/
	#define REENT_POST				reentrant	/* KEIL C251 needs to be told about re-entrance		*/

	#pragma REGPARMS							/* Use registers to pass arguments					*/
	typedef long intptr_t;						/* Keil: no stdint.h. Ptr are 3/4 bytes				*/
	typedef long int32_t;						/* Same here										*/
	typedef unsigned long uint32_t;				/* Same here										*/

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/

/* ------------------------------------------------------------------------------------------------ */
/* 8051 / SDCC																						*/
/* ------------------------------------------------------------------------------------------------ */

#elif defined(SDCC) &&  defined(SDCC_mcs51)
	#if ((OS_STATIC_STACK)==0)					/* Does not work with malloc()						*/
	  #error "Abassi: 8051 with SDCC MUST use a static stack memory pool using OS_STATIC_STACK"
	  #error "Abassi: Must be sized to OS_STATIC_TASK*512 + 255"
	#endif

	#define OX_AE_STACK_SIZE		0			/* Most ports don't need a stack for Adam & Eve		*/
	#define OX_BSS_ZERO				0			/* SDCC does not zero the BSS for the 8051			*/
	#define OX_MEMSET_LIB			1			/* The function call is more efficient than in-line	*/
	#define OX_N_INTERRUPTS			7			/* Basic 8051 has 6 interrupts (+1 extra as example)*/
	#define OX_USE_STRLIB			1			/* The function call is more efficient than in-line	*/
	#define printf printf_small

	typedef char OSalign_t;						/* Data type that will use the alignment size		*/
#else
	#error "Abassi: Unsupported processor/compiler"

#endif

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU ARE SURE YOU KNOW WHAT YOU ARE DOING			*/
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

/* ------------------------------------------------------------------------------------------------ */
/* Build options validation																			*/
/* ------------------------------------------------------------------------------------------------ */

#define OX_N_CORE 	1							/* Some add-on are used for both single & multicore	*/

#ifndef OS_ALLOC_SIZE
  #error "Abassi: The build option OS_ALLOC_SIZE hasn't been defined"
#elif ((OS_ALLOC_SIZE) < 0)
  #error "Abassi: The build option OS_ALLOC_SIZE has a negative value"  
  #error "        >= 0 as defines the number bytes allocated for the OS memory pool"
  #error "        == 0 if malloc() is preferred from OSalloc()"
#endif
#ifndef OS_COOPERATIVE
  #error "Abassi: The build option OS_COOPERATIVE hasn't been defined"
#endif
#ifndef OS_EVENTS
  #error "Abassi: The build option OS_EVENTS hasn't been defined"
#endif
#ifndef OS_FCFS
  #error "Abassi: The build option OS_FCFS hasn't been defined"
#endif
#ifndef OS_IDLE_STACK
  #error "Abassi: The build option OS_IDLE_STACK hasn't been defined"
#elif ((OS_IDLE_STACK) < 0)
  #error "Abassi: The build option OS_IDLE_STACK has a negative value"
  #error "        >= 0 defines the number bytes allocated for the Idle Task stack"
  #error "        == 0 the application supplied its own Idle Task"
#endif
#ifndef OS_LOGGING_TYPE
  #error "Abassi: The build option OS_LOGGING_TYPE hasn't been defined"
#elif ((OS_LOGGING_TYPE) < 0)
  #error "Abassi: The build option OS_LOGGING_TYPE is negative"
  #error "       = 0 disable logging"
  #error "       = 1 specifies to perform direct formatted print to OSputChar()"
  #error "       > 1 specifies the buffer size (in # of messages) to accumulate past logging messages"
#endif
#ifndef OS_MAILBOX
  #error "Abassi: The build option OS_MAILBOX hasn't been defined"
#endif
#ifndef OS_MAX_PEND_RQST
  #error "Abassi: The build option OS_MAX_PEND_RQST hasn't been defined"
#elif ((OS_MAX_PEND_RQST) < 2U)
  #error "Abassi: The build option OS_MAX_PEND_RQST must be > 1"
  #error "        It defines the size of the queue to absorb kernel requests during interrupts"
#endif
#ifndef OS_MIN_STACK_USE
  #error "Abassi: The build option OS_MIN_STACK_USE hasn't been defined"
#endif
#ifndef OS_MTX_DEADLOCK
  #error "Abassi: The build option OS_MTX_DEADLOCK hasn't been defined"
#endif
#ifndef OS_MTX_INVERSION
  #error "Abassi: The build option OS_MTX_INVERSION hasn't been defined"
#endif
#ifndef OS_NAMES
  #error "Abassi: The build option OS_NAMES hasn't been defined"
#endif
#ifndef OS_NESTED_INTS
  #error "Abassi: The build option OS_NESTED_INTS hasn't been defined"
#endif
#ifndef OS_PRIO_CHANGE
  #error "Abassi: The build option OS_PRIO_CHANGE hasn't been defined"
#endif
#ifndef OS_PRIO_MIN
  #error "Abassi: The build option OS_PRIO_MIN hasn't been defined"
#elif ((OS_PRIO_MIN) < 0)
  #error "Abassi: The build option OS_PRIO_MIN is negative"
  #error "        The highest priority value is 0"
  #error "        The lowest priority must have a value > 0"
#endif
#ifndef OS_PRIO_SAME
  #error "Abassi: The build option OS_PRIO_SAME hasn't been defined"
#endif
#ifndef OS_ROUND_ROBIN
  #error "Abassi: The build option OS_ROUND_ROBIN hasn't been defined"
#endif
#ifndef OS_RUNTIME
  #error "Abassi: The build option OS_RUNTIME hasn't been defined"
#endif
#ifndef OS_SEARCH_ALGO
  #error "Abassi: The build option OS_SEARCH_ALGO hasn't been defined"
#endif
#ifndef OS_STARVE_PRIO
  #error "Abassi: The build option OS_STARVE_PRIO hasn't been defined"
#endif
#ifndef OS_STARVE_RUN_MAX
  #error "Abassi: The build option OS_STARVE_RUN_MAX hasn't been defined"
#endif
#ifndef OS_STARVE_WAIT_MAX
  #error "Abassi: The build option OS_STARVE_WAIT_MAX hasn't been defined"
#endif
#ifdef OS_STATIC_BUF_MBLK
 #if ((OS_STATIC_BUF_MBLK) < 0)
  #error "Abassi: The build option OS_STATIC_BUF_MBLK is negative"
  #error "        Must be >= 0 as it defines the buffer memory to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
 #endif
#endif
#ifdef OS_STATIC_MBLK
 #if ((OS_STATIC_MBLK) < 0)
  #error "Abassi: The build option OS_STATIC_MBLK is negative"
  #error "        Must be >= 0 as it defines the number memory block descriptors to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
 #endif
#endif
#ifndef OS_STATIC_BUF_MBX
  #error "Abassi: The build option OS_STATIC_BUF_MBX hasn't been defined"
#elif ((OS_STATIC_BUF_MBX) < 0)
  #error "Abassi: The build option OS_STATIC_BUF_MBX is negative"
  #error "        This defines the number of mailbox elements to allocate"
  #error "        It must be greater or equal to the sum of all mailbox buffer size"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_STATIC_MBX
  #error "Abassi: The build option OS_STATIC_MBX hasn't been defined"
#elif ((OS_STATIC_MBX) < 0)
  #error "Abassi: The build option OS_STATIC_MBX is negative"
  #error "        Must be >= 0 as it defines the number mailbox descriptors to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#if ((OS_STATIC_MBX) == 0) && ((OS_STATIC_BUF_MBX) > 0)
  #error "Abassi: The build option OS_STATIC_BUF_MBX is non-zero when OS_STATIC_MBX` is zero"
  #error "        OS_STATIC_BUF_MBX takes into account the number of mailboxes in the application"
  #error "        to allocate the correct amount of memory"
  #error "        Set OS_STATIC_MBX to the number of mailboxes in the application"
#endif
#ifndef OS_STATIC_NAME
  #error "Abassi: The build option OS_STATIC_NAME hasn't been defined"
#elif ((OS_STATIC_NAME) < 0)
  #error "Abassi: The build option OS_STATIC_NAME is negative"
  #error "        Must be >= 0 as it defines the number of bytes allocated for the names"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_STATIC_SEM
  #error "Abassi: The build option OS_STATIC_SEM hasn't been defined"
#elif ((OS_STATIC_SEM) < 0)
  #error "Abassi: The build option OS_STATIC_SEM is negative"
  #error "        Must be >= 0 as it defines the number mailbox descriptors to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_STATIC_STACK
  #error "Abassi: The build option OS_STATIC_STACK hasn't been defined"
#elif ((OS_STATIC_STACK) < 0)
  #error "Abassi: The build option OS_STATIC_STACK is negative"
  #error "        Must be >=0 as it defines the total number of bytes allocated for all task stacks"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_STATIC_TASK
  #error "Abassi: The build option OS_STATIC_TASK hasn't been defined"
#elif ((OS_STATIC_TASK) < 0)
  #error "Abassi: The build option OS_STATIC_TASK is negative"
  #error "        Must be >= 0 as it defines the number task descriptors to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_STATIC_TIM_SRV
  #error "Abassi: The build option OS_STATIC_TIM_SRV hasn't been defined"
#elif ((OS_STATIC_TIM_SRV) < 0)
  #error "Abassi: The build option OS_STATIC_TIM_SRV is negative"
  #error "        Must be >= 0 as it defines the number timer service descriptors to allocate"
  #error "        If set to 0, memory is obtained through OSalloc()"
#endif
#ifndef OS_TASK_SUSPEND
  #error "Abassi: The build option OS_TASK_SUSPEND hasn't been defined"
#endif
#ifndef OS_TIMEOUT
  #error "Abassi: The build option OS_TIMEOUT hasn't been defined"
#endif
#ifndef OS_TIMER_CB
  #error "Abassi: The build option OS_TIMER_CB hasn't been defined"
#elif ((OS_TIMER_CB) < 0)
  #error "Abassi: The build option OS_TIMER_CB is negative"
  #error "        Must be >= 0 as it defines the rate the timer tick calls the callback function"
  #error "        If set to 0, the timer tick does not call the callback function"
#endif
#ifndef OS_TIMER_SRV
  #error "Abassi: The build option OS_TIMER_SRV hasn't been defined"
#endif
#ifndef OS_TIMER_US
  #error "Abassi: The build option OS_TIMER_US hasn't been defined"
#elif ((OS_TIMER_US) < 0)
  #error "Abassi: The build option OS_TIMER_US is negative"
  #error "        Must be positive (>0) as it defines the period of the timer tick"
#endif
#ifndef OS_USE_TASK_ARG
  #error "Abassi: The build option OS_USE_TASK_ARG hasn't been defined"
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Internal overloading																				*/
/* ------------------------------------------------------------------------------------------------ */

#if (OS_COOPERATIVE)							/* When the kernel operates in a cooperative mode	*/
	#define OX_ROUND_ROBIN		0				/* Round robin cannot exist							*/
	#define OX_SEARCH_ALGO		(-1)			/* The search algorithm must use the linked list	*/
#endif


#if (OS_TIMER_US)								/* If the timer is used, a few human definitions   	*/
  #define OS_TICK_PER_SEC	((int)(((((long)(OS_TIMER_US))/2)+1000000L)       /((long)(OS_TIMER_US))))
  #define OS_MS_TO_TICK(x)	((int)(((((long)(OS_TIMER_US))/2)+((x)*   1000L)) /((long)(OS_TIMER_US))))
  #define OS_SEC_TO_TICK(x) ((int)(((((long)(OS_TIMER_US))/2)+((x)*1000000L)) /((long)(OS_TIMER_US))))
  #define OS_HMS_TO_TICK(h,m,s)	(OS_SEC_TO_TICK(((h)*60+(m))*60+(s)))
  #if ((OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0))/* Round Robin value must be >= Timer us			*/
	#if ((OS_ROUND_ROBIN) < 0)					/* Need to have the absolute value					*/
	  #define OS_RR_ABS 		(0-(OS_ROUND_ROBIN))
	#else
	  #define OS_RR_ABS 		(OS_ROUND_ROBIN)
	#endif
    #if ((OS_RR_ABS) < (OS_TIMER_US))			/* Force it == if too small							*/
	  #if ((OS_ROUND_ROBIN) > 0)
		#define OX_ROUND_ROBIN	(OS_TIMER_US)
	  #else
		#define OX_ROUND_ROBIN	(0-(OS_TIMER_US))
	  #endif
	  #define OX_RR_ABS 		(OX_ROUND_ROBIN)
	#endif										/* Number of ticks in a round robin time slice		*/
	#ifndef OX_RR_ABS
	  #define OX_RR_ABS			(OS_RR_ABS)
	#endif
	#define OX_RR_TICK			((OX_RR_ABS)/(OS_TIMER_US))
  #endif
#else
  #if ((OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0))	/* OS_COOPERATIVE overloads OS_ROUND_ROBIN		*/
	#error "Abassi: The build option OS_ROUND_ROBIN is true, but OS_TIMER_US is false"
	#error "        This means round robin is activated but the timer is not enable"
  #endif
  #if (OS_STARVE_WAIT_MAX)
	#error "Abassi: The build option OS_STARVE_WAIT_MAX is true, but OS_TIMER_US is false"
	#error "        This means starvation priority boosting is activated but the timer is not enable"
  #endif
  #if ((OS_TIMEOUT) > 0)
	#error "Abassi: The build option OS_TIMEOUT is true, but OS_TIMER_US is false"
	#error "        This means timeouts are activated but the timer is not enable"
  #endif
  #if (OS_TIMER_CB)
	#error "Abassi: The build option OS_TIMER_CB is true, but OS_TIMER_US is false"
	#error "        This means a timer callback function is supplied but the timer is not enable"
  #endif
  #if (OS_TIMER_SRV)
	#error "Abassi: The build option OS_TIMER_SRV is true, but OS_TIMER_US is false"
	#error "        This means the timer services are enable but the timer is not enable"
  #endif
#endif


#if ((OS_STATIC_TASK) && ((OS_RUNTIME)==0))
  #error "Abassi: The build option OS_STATIC_TASK is true when OS_RUNTIME is false"
  #error "        OS_STATIC_TASK can only be used if OS_RUNTIME is true"
#endif
#if ((OS_STATIC_STACK) && ((OS_RUNTIME)==0))
  #error "Abassi: The build option OS_STATIC_STACK is true when OS_RUNTIME is false"
  #error "        OS_STATIC_STACK can only be used if OS_RUNTIME is true"
#endif
#if ((OS_STATIC_SEM) && ((OS_RUNTIME)==0))
  #error "Abassi: The build option OS_STATIC_SEM is true when OS_RUNTIME is false"
  #error "        OS_STATIC_SEM can only be used if OS_RUNTIME is true"
#endif
#if ((OS_STATIC_MBX) && ((OS_RUNTIME)==0) && (OS_MAILBOX))
  #error "Abassi: The build option OS_STATIC_MBX is true when OS_RUNTIME is false"
  #error "        OS_STATIC_MBX can only be used if OS_RUNTIME is true"
#endif
#if ((OS_STATIC_BUF_MBX) && ((OS_RUNTIME)==0) && (OS_MAILBOX))
  #error "Abassi: The build option OS_STATIC_BUF_MBX is true when OS_RUNTIME is false"
  #error "        OS_STATIC_BUF_MBX can only be used if OS_RUNTIME is true"
#endif
#ifdef OS_MEM_BLOCK
  #ifndef OS_STATIC_MBLK
	#error "Abassi: The build option OS_MEM_BLOCK is defined and OS_STATIC_MBLK"
	#error "        OS_STATIC_MBLK must be defined when OS_MEM_BLOCK is defined"
  #endif
  #ifndef OS_STATIC_BUF_MBLK
	#error "Abassi: The build option OS_MEM_BLOCK is defined and OS_STATIC_BUF_MBLK"
	#error "        OS_STATIC_BUF_MBLK must be defined when OS_MEM_BLOCK is defined"
  #endif
 #if (OS_MEM_BLOCK) && ((OS_RUNTIME)==0)
  #error "Abassi: The build option OS_MEM_BLOCK is true when OS_RUNTIME is false"
  #error "        Memory block can only be created at run time, set OS_RUNTIME to non-zero"
 #endif
 #if ((OS_STATIC_MBLK) && ((OS_RUNTIME)==0) && (OS_MEM_BLOCK))
  #error "Abassi: The build option OS_STATIC_MBLK is true when OS_RUNTIME is false"
  #error "        OS_STATIC_MBLK can only be used if OS_RUNTIME is true"
 #endif
 #if ((OS_STATIC_BUF_MBLK) && ((OS_RUNTIME)==0) && (OS_MEM_BLOCK))
  #error "Abassi: The build option OS_STATIC_BUF_MBLK is true when OS_RUNTIME is false"
  #error "        OS_STATIC_BUF_MBLK can only be used if OS_RUNTIME is true"
 #endif
#endif
#if ((OS_STATIC_TIM_SRV) && ((OS_RUNTIME)==0) && (OS_TIMER_SRV))
  #error "Abassi: The build option OS_STATIC_SEM is true when OS_RUNTIME is false"
  #error "        OS_STATIC_SEM can only be used if OS_RUNTIME is true"
#endif


#if ((OS_LOGGING_TYPE)==1)						/* For ASCII dump, need named descriptors			*/
  #define OX_NAMES				-1				/* So override the user supplied definition			*/
#endif


#if ((OS_TASK_SUSPEND) || (OS_MTX_DEADLOCK) || (OS_MTX_INVERSION))
  #define OX_HANDLE_MTX			1				/* These 3 build options require info about mutex	*/
#else											/* So override the user supplied definition			*/
  #define OX_HANDLE_MTX			0
#endif


#if ((OS_MTX_INVERSION) || (OS_STARVE_WAIT_MAX))
  #define OX_PRIO_CHANGE		1				/* These 3 build options require changing task prio	*/
#endif											/* So override the user supplied definition			*/


#if ((OS_MAILBOX)==0)							/* Mailboxes are not part of the build				*/
  #define OX_STATIC_MBX			0				/* Override the user supplied definitions			*/
  #define OX_STATIC_BUF_MBX 	0
#endif

#ifndef OS_STATIC_BUF_MBLK						/* Optional build option, set to false if not def'd	*/
  #define OS_STATIC_BUF_MBLK	0
#endif
#ifndef OS_STATIC_MBLK							/* Optional build option, set to false if not def'd	*/
  #define OS_STATIC_MBLK		0
#endif
#ifndef OS_MEM_BLOCK							/* Optional build option, set to false if not def'd	*/
  #define OS_MEM_BLOCK			0
#endif
#if ((OS_MEM_BLOCK)==0)							/* Memory Blocks are not part of the build			*/
  #define OX_STATIC_MBLK		0				/* Override the user supplied definitions			*/
  #define OX_STATIC_BUF_MBLK 	0
#endif

#ifdef OX_NAMES									/* Names are not part of the build					*/
 #if ((OX_NAMES) <= 0)
  #define OX_STATIC_NAME		0				/* Names not supported, no need to reserve memory	*/
 #endif
#else
 #if ((OS_NAMES) <= 0)
  #define OX_STATIC_NAME		0				/* Names not supported, no need to reserve memory	*/
 #endif
#endif


#if ((OS_STARVE_WAIT_MAX) && ((OS_STARVE_PRIO) == (OS_PRIO_MIN)))
  #define OX_STARVE_WAIT_MAX	0				/* When set to min prio, is turned OFF				*/
#else
  #define OX_STARVE_WAIT_MAX	(OS_STARVE_WAIT_MAX)
#endif

#if ((OX_STARVE_WAIT_MAX) || ((OS_TIMER_US) && ((OS_ROUND_ROBIN) && ((OS_COOPERATIVE)==0))))
  #define OX_PRIO_SAME			1				/* Starvation protection requires same prio tasks	*/
#endif

#if (OX_STARVE_WAIT_MAX)
  #if ((OS_STARVE_PRIO) < 0)
	#if (((-(OS_STARVE_PRIO)) > (OS_PRIO_MIN)) || ((-(OS_STARVE_PRIO)) < 0))
	  #error "Abassi: The build option OS_STARVE_PRIO is out of range"
	  #error "        Must be bounded between 0 and OS_PRIO_MIN"
	#endif
  #else
	#if (((OS_STARVE_PRIO) > (OS_PRIO_MIN)) || ((OS_STARVE_PRIO) < 0))
	  #error "Abassi: The build option OS_STARVE_PRIO is out of range"
	  #error "        Must be bounded between 0 and OS_PRIO_MIN"
	#endif
  #endif
  #if ((OS_STARVE_RUN_MAX) == 0)
	#error "Abassi: With starvation protection enabled, OS_STARVE_RUN_MAX cannot be set to zero"
	#error "        Starvation protection is enabled because OS_STARVE_WAIT_MAX is non-zero"
  #endif
#else
	#define OX_STARVE_PRIO		0				/* Starvation protect is not part of the build		*/
	#define OX_STARVE_RUN_MAX	0				/* Override the user supplied definitions			*/
#endif

#if ((OS_TIMER_SRV) == 0)						/* Timer services are not part of the build			*/
  #define OX_STATIC_TIM_SRV 	0				/* Override the user supplied definitions			*/
#endif


#if (OS_MIN_STACK_USE)							/* Control of the kernel stack usage				*/
  #define OX_STATIC 			static 
#else
  #define OX_STATIC
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Overloading of build definitions																	*/
/* All build options are remapped to OX_XXX to avoid confusion when using the tokens				*/
/* From now on, all build options are OX_XXX														*/
/* ------------------------------------------------------------------------------------------------ */

#ifndef OS_AE_STACK_SIZE
  #define OS_AE_STACK_SIZE		0
#endif
#ifndef OX_AE_STACK_SIZE
  #define OX_AE_STACK_SIZE		(OS_AE_STACK_SIZE)
#endif
#ifndef OX_ALLOC_SIZE
  #define OX_ALLOC_SIZE			(OS_ALLOC_SIZE)
#endif
#ifndef OX_COOPERATIVE
  #define OX_COOPERATIVE		(OS_COOPERATIVE)
#endif
#ifndef OX_EVENTS
  #define OX_EVENTS				(OS_EVENTS)
#endif
#ifndef OX_FCFS
  #define OX_FCFS				(OS_FCFS)
#endif
#ifndef OX_IDLE_STACK
  #define OX_IDLE_STACK			(OS_IDLE_STACK)
#endif
#ifndef OX_INT_1_READ
  #define OX_INT_1_READ			1
#endif
#ifndef OX_ISR_TBL_OFFSET
  #define OX_ISR_TBL_OFFSET		0
#endif
#ifndef OX_LOGGING_TYPE
  #define OX_LOGGING_TYPE		(OS_LOGGING_TYPE)
#endif
#ifndef OX_MAILBOX
  #define OX_MAILBOX			(OS_MAILBOX)
#endif
#ifndef OX_MAX_PEND_RQST
  #define OX_MAX_PEND_RQST		(OS_MAX_PEND_RQST)
#endif
#ifndef OX_MEM_BLOCK
  #define OX_MEM_BLOCK			(OS_MEM_BLOCK)
#endif
#ifndef OX_MIN_STACK_USE
  #define OX_MIN_STACK_USE		(OS_MIN_STACK_USE)
#endif
#ifndef OX_MTX_DEADLOCK
  #define OX_MTX_DEADLOCK		(OS_MTX_DEADLOCK)
#endif
#ifndef OX_MTX_INVERSION
  #define OX_MTX_INVERSION		(OS_MTX_INVERSION)
#endif
#if (((OX_MTX_INVERSION) > 999) || ((OX_MTX_INVERSION) < -999))
  #define OX_PRIO_INV_ON_OFF	1
#else
  #define OX_PRIO_INV_ON_OFF	0
#endif
#ifndef OX_N_INTERRUPTS
  #define OX_N_INTERRUPTS		(OS_N_INTERRUPTS)
#endif
#ifndef OX_NESTED_INTS
  #define OX_NESTED_INTS		(OS_NESTED_INTS)
#endif
#ifndef OX_NAMES
  #define OX_NAMES				(OS_NAMES)
#endif
#ifndef OS_OUT_OF_MEM								/* Optional build oiption						*/
  #define OX_OUT_OF_MEM			0
#else
  #define OX_OUT_OF_MEM			(OS_OUT_OF_MEM)
#endif
#ifndef OX_PRIO_CHANGE
  #define OX_PRIO_CHANGE		(OS_PRIO_CHANGE)
#endif
#ifndef OX_PRIO_MIN
  #define OX_PRIO_MIN			(OS_PRIO_MIN)
#endif
#ifndef OX_PRIO_SAME
  #define OX_PRIO_SAME			(OS_PRIO_SAME)
#endif
#ifndef OX_ROUND_ROBIN
  #define OX_ROUND_ROBIN		(OS_ROUND_ROBIN)
#endif
#ifndef OX_RUNTIME
  #define OX_RUNTIME			(OS_RUNTIME)
#endif
#ifndef OX_SEARCH_ALGO
  #define OX_SEARCH_ALGO		(OS_SEARCH_ALGO)
#endif
#ifndef OS_STACK_CHECK								/* Optional build oiption						*/
  #define OX_STACK_CHECK		0
#else
  #define OX_STACK_CHECK		(OS_STACK_CHECK)
#endif
#ifndef OX_STARVE_PRIO
  #define OX_STARVE_PRIO		(OS_STARVE_PRIO)
#endif
#ifndef OX_STARVE_RUN_MAX
  #define OX_STARVE_RUN_MAX		(OS_STARVE_RUN_MAX)
#endif
#ifndef OX_STATIC_BUF_MBX
  #define OX_STATIC_BUF_MBX		(OS_STATIC_BUF_MBX)
#endif
#ifndef OX_STATIC_MBX
  #define OX_STATIC_MBX			(OS_STATIC_MBX)
#endif
#ifndef OX_STATIC_BUF_MBLK
  #define OX_STATIC_BUF_MBLK	(OS_STATIC_BUF_MBLK)
#endif
#ifndef OX_STATIC_MBLK
  #define OX_STATIC_MBLK		(OS_STATIC_MBLK)
#endif
#ifndef OX_STATIC_NAME
  #define OX_STATIC_NAME		(OS_STATIC_NAME)
#endif
#ifndef OX_STATIC_SEM
  #define OX_STATIC_SEM			(OS_STATIC_SEM)
#endif
#ifndef OX_STATIC_STACK
  #define OX_STATIC_STACK		(OS_STATIC_STACK)
#endif
#ifndef OX_STATIC_TASK
  #define OX_STATIC_TASK		(OS_STATIC_TASK)
#endif
#ifndef OX_STATIC_TIM_SRV
  #define OX_STATIC_TIM_SRV		(OS_STATIC_TIM_SRV)
#endif
#ifndef OX_TASK_SUSPEND
  #define OX_TASK_SUSPEND		(OS_TASK_SUSPEND)
#endif
#ifndef OX_TASK_XTRA_FIELD
  #define OX_TASK_XTRA_FIELD	0
#endif
#ifdef OS_TIM_TICK_ACK
 #ifndef OX_TIM_TICK_ACK
  #define OX_TIM_TICK_ACK()		OS_TIM_TICK_ACK()
 #endif
#endif
#ifndef OX_TIMEOUT
  #define OX_TIMEOUT			(OS_TIMEOUT)
#endif
#ifndef OX_TIMER_CB
  #define OX_TIMER_CB			(OS_TIMER_CB)
#endif
#ifndef OX_TIMER_SRV
  #define OX_TIMER_SRV			(OS_TIMER_SRV)
#endif
#ifndef OX_TIMER_US
  #define OX_TIMER_US			(OS_TIMER_US)
#endif
#ifndef OX_USE_INT_REGISTER
  #define OX_USE_INT_REGISTER	0
#endif
#ifndef OX_USE_TASK_ARG
  #define OX_USE_TASK_ARG		(OS_USE_TASK_ARG)
#endif

#ifndef OX_STR_CONST
  typedef const char OSstrCnst_t;
#endif

#ifndef REENT_POST
  #define REENT_POST
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Data types & definitions																			*/
/* ------------------------------------------------------------------------------------------------ */

/* ------------------------------------------------	*/
/* Semaphore descriptor								*/

typedef struct _SEM_t {							/* Semaphore descriptor								*/
	struct _TSK_t *Blocked;						/* Linked list of tasks blocked by this semaphore	*/
	struct _TSK_t *MtxOwner;					/* Owner / locker of the mutex (when it is a mutex)	*/
  #if (OX_HANDLE_MTX)
	struct _SEM_t *MtxNext;						/* The next Mutex owned by the same task			*/
  #endif
  #if (OX_NAMES)
	const char    *SemName;						/* Name of the semaphore							*/
	struct _SEM_t *SemaNext;					/* Next semaphore in system (when searching name)	*/
  #endif
	int   Value;								/* Semaphore counter								*/
  #if (OX_FCFS)									/* When 1st come 1st served enable					*/
	int IsFCFS;									/* This indicates if it's a FCFS semaphore/mutex	*/
  #endif
  #if ((OX_MTX_INVERSION) < 0)					/* For priority ceiling, we need to attach a		*/
	int CeilPrio;								/* priority to the mutex							*/
  #endif
  #if (OX_PRIO_INV_ON_OFF)
	int PrioInvOn;
  #endif
} SEM_t;

/* ------------------------------------------------	*/
/* Mutex descriptor									*/

typedef SEM_t MTX_t;							/* Mutexes are same data type as semaphores			*/
												/* Would have prefer to use .field=XXX but a few	*/
#if ((OX_RUNTIME) >= 0)							/* compilers do not support it. So has to revert	*/
  #if (OX_HANDLE_MTX)							/* to the old K&R initialization scheme				*/
	#define INIT_SEM_MTXNEXT			,(SEM_t *)NULL
  #else
	#define INIT_SEM_MTXNEXT
  #endif
  #if (OX_NAMES)
	#define INIT_SEM_NAME(_Name)		,(_Name)
	#define INIT_SEM_SEMANEXT			,(SEM_t *)NULL
  #else
	#define INIT_SEM_NAME(_Name)
	#define INIT_SEM_SEMANEXT
  #endif
  #if (OX_FCFS)
	#define INIT_SEM_ISFCFS				,0
  #else
	#define INIT_SEM_ISFCFS
  #endif
  #if ((OX_MTX_INVERSION) < 0)
	#define INIT_SEM_CEILPRIO			,0
	#define INIT_MTX_CEILPRIO			,(OX_PRIO_MIN)
  #else
	#define INIT_SEM_CEILPRIO
	#define INIT_MTX_CEILPRIO
  #endif
  #if (OX_PRIO_INV_ON_OFF)
	#define INIT_SEM_PRIO_INV_ON		,1
  #else
	#define INIT_SEM_PRIO_INV_ON
  #endif
  #define SEM_STATIC(__Var, __Name)		\
   static SEM_t __Var##__ = {			\
	 (TSK_t *)NULL						\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(__Name)				\
	INIT_SEM_SEMANEXT					\
	,0									\
	INIT_SEM_ISFCFS						\
	INIT_SEM_CEILPRIO					\
	INIT_SEM_PRIO_INV_ON				\
  };									\
  SEM_t *(__Var) = &(__Var##__)

  #define MTX_STATIC(__Var, __Name)		\
   static MTX_t __Var##__ = {			\
	 (TSK_t *)NULL						\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(__Name)				\
	INIT_SEM_SEMANEXT					\
	,1									\
	INIT_SEM_ISFCFS						\
	INIT_MTX_CEILPRIO					\
	INIT_SEM_PRIO_INV_ON				\
  };									\
  MTX_t *(__Var) = &(__Var##__)
#endif

/* ------------------------------------------------ */
/* Timer Services descriptor						*/

#if (OX_TIMER_SRV)

typedef void(*OStimerFct_t)(intptr_t Arg);		/* Function attached when Ops == TIM_CALL_FCT		*/

  typedef struct _TIM_t {
	struct _TIM_t *TmNext;						/* Next timer descriptor in the linked list			*/
	struct _TIM_t *TmPrev;						/* Previous timer descriptor in the linked list		*/
  #if (OX_NAMES)
	const char    *TimName;						/* Name of a timer service							*/
	struct _TIM_t *TimNext;						/* Next timer in the list (to search name)			*/
  #endif
	void          *Ptr;							/* Ptr to fct, data or RTOS component				*/
	void (*FctPtr)(intptr_t Arg);				/* *Ptr to fct call upon timeout					*/
	intptr_t       Arg;							/* When operation needs an argument					*/
	unsigned int   Ops;							/* Operation to perform								*/
	int            Period;						/* When != 0, period of a periodic timer			*/
	int            TickDone;					/* Tick value for timeout							*/
	int            ToAdd;						/* Increment, decrement to add to argument			*/
  } TIM_t;

 #if ((OX_RUNTIME) >= 0)
  #if (OX_NAMES)
	#define INIT_TIM_NAME(_Name)		,(_Name)
	#define INIT_TIM_TIMNEXT			,(TIM_t *)NULL
  #else
	#define INIT_TIM_NAME(_Name)
	#define INIT_TIM_TIMNEXT
  #endif

  #define TIM_STATIC(__Var, __Name)		\
   static TIM_t __Var##__ = {			\
	  (TIM_t *)NULL						\
	, (TIM_t *)NULL						\
	INIT_TIM_NAME(__Name)				\
	INIT_TIM_TIMNEXT					\
	, (void *)NULL						\
	, (OStimerFct_t)NULL				\
	, (intptr_t)0						\
	, 0U								\
	, 0									\
	, 0									\
	, 0									\
   };									\
   TIM_t *(__Var) = &(__Var##__)
 #endif

#endif

/* ------------------------------------------------	*/
/* Task descriptor									*/

typedef struct _TSK_t {							/* Process descriptor								*/
	void       *Stack;							/* Stack pointer when not running (context save)	*/
  #if (OX_TASK_XTRA_FIELD)						/* Some ports require custom information in here	*/
	intptr_t    XtraData[OX_TASK_XTRA_FIELD];
  #endif
	SEM_t       Priv;							/* Private semaphore								*/
	SEM_t      *Blocker;						/* Semaphore blocking this task						*/
	struct _TSK_t *Next;						/* Next task blocked/ready to run					*/
	struct _TSK_t *Prev;						/* Previous task blocked/ready to run				*/
  #if (OX_PRIO_SAME)
	struct _TSK_t *Last;						/* Last ready with same priority: eliminates a scan	*/
  #endif
  #if (OX_NAMES)
	const char    *TskName;						/* Name of the task									*/
	struct _TSK_t *TaskNext;					/* Next task in the list (to search name)			*/
  #endif
  #if (OX_HANDLE_MTX)
	SEM_t *MyMutex;								/* Linked list of all mutex I'm the owner			*/
  #endif 
  #if (OX_USE_TASK_ARG)
	void *TaskArg;								/* Pointer to the argument of the task				*/
  #endif
  #if ((OX_SEARCH_ALGO) < 0)					/* When using link lists instead of array			*/
	struct _TSK_t *PrioNext;					/* Task at a lower priority							*/
	struct _TSK_t *PrioPrev;					/* Task at a higher priority						*/
  #endif
  #if (OX_STARVE_WAIT_MAX)
	struct _TSK_t *StrvNext;					/* Next in linked list of ready to run task			*/
	struct _TSK_t *StrvPrev;					/* Previous in linked list of ready to run task		*/
	struct _TSK_t *StrvLast;					/* Last in the linked list of ready to run			*/
	volatile int   StrvState;					/* State of the protection for this task			*/
	int            StrvPrio;					/* Priority before starvation priority promotion	*/
	int            StrvTimOut;					/* Timer tick value to increase the task priority	*/
  #endif
  #if ((OX_STARVE_PRIO) < 0)
	int  StrvPrioMax;							/* Maximum priority under starvation				*/
  #endif
  #if ((OX_STARVE_RUN_MAX) < 0)
	int StrvRunMax;								/* Maximum run-time under starvation				*/
  #endif
  #if ((OX_STARVE_WAIT_MAX) < 0)
	int StrvWaitMax;							/* Maximum time at same priority under starvation	*/
  #endif
  #if ((OX_TIMEOUT) > 0)
	struct _TSK_t *ToutNext;					/* Next task in the timeout linked list				*/
	struct _TSK_t *ToutPrev;					/* Previous task in the timeout linked list			*/
	int            TickTime;					/* RTOS counter tick value for timeout				*/
  #endif
  #if ((OX_TASK_SUSPEND) || ((OX_TIMEOUT) > 0))
	int  PrevRetVal;							/* Return value before getting suspended			*/
  #endif
	int         Prio;							/* Current priority of the task						*/
  #if (OX_TASK_SUSPEND)
	unsigned int SuspRqst;						/* Request to suspend the task (Boolean)			*/
  #endif
  #if (OX_ROUND_ROBIN)
	unsigned int RRcnt;							/* This counts how many timer ticks the task has	*/
	         int RRtick;						/* been running, RRtick is last timer tick update	*/
	#if ((OX_ROUND_ROBIN) < 0)
	  unsigned int RRmax;						/* Variable round-robin time slice duration			*/
	#endif										/* Round-robin maximum run-time slice duration		*/
  #endif
  #if (OX_COOPERATIVE)
	int Yield;									/* Request to yield (3 states)						*/
  #endif
  #if (OX_MTX_INVERSION)						/* To eliminate priority inversion, need to			*/
	volatile int OriPrio;						/* temporary increase the priority of the task		*/
  #endif										/* Need volatile as one port has problems without	*/
  #if (OX_EVENTS)
	unsigned int EventAcc;						/* Accumulation of the events bits					*/
	unsigned int EventRX;						/* Event bits received								*/
	unsigned int EventAND;						/* AND mask for the events to unblock				*/
	unsigned int EventOR;						/* OR mask for the events to unblock				*/
  #endif
  #if (OX_MAILBOX)
	intptr_t       MboxMsg;						/* Message to write when blocked on a full mailbox	*/
  #endif
  #if (OX_MEM_BLOCK)
	OSalign_t     *MblkBuf;						/* Buffer to return when blocked on an empty pool	*/
  #endif
  #if (OX_STACK_CHECK)							/* When stack checking, memo stack boundaries		*/
	int32_t *StackLow;							/* Low memory address of the stack					*/
	int32_t *StackHigh;							/* High memory address of the stack					*/
  #endif
} TSK_t;

#if ((OX_RUNTIME) >= 0)
  typedef struct {								/* When Data ptr are smaller size than function	ptr */
	TSK_t Tsk;									/* then we can't re-use a data pointer to hold the	*/
	void (*TskFct)(void);						/* address of the function attached to the task in	*/
  } _sTSK_t;									/* anything in the task descriptor. e.g MSP430X		*/

  #if (OX_TASK_XTRA_FIELD)
   #if ((OX_TASK_XTRA_FIELD)   == 1)
	#define INIT_TASK_XTRADATA			,{((intptr_t)0)}
   #elif ((OX_TASK_XTRA_FIELD) == 2)
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0)}
   #elif ((OX_TASK_XTRA_FIELD) == 3)
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0)}
   #elif ((OX_TASK_XTRA_FIELD) == 4)
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0),((intptr_t)0)}
   #elif ((OX_TASK_XTRA_FIELD) == 5)
	#define INIT_TASK_XTRADATA			,{((intptr_t)0),((intptr_t)0),((intptr_t)0),((intptr_t)0),	\
										 ((intptr_t)0)}
   #endif
  #else
	#define INIT_TASK_XTRADATA
  #endif
  #if (OX_PRIO_SAME)
	#define INIT_TASK_LAST				,(TSK_t *)NULL
  #else
	#define INIT_TASK_LAST
  #endif
  #if (OX_NAMES)
	#define INIT_TASK_NAME(_Name)		,(_Name)
	#define INIT_TASK_TASKNEXT			,(TSK_t *)NULL
  #else
	#define INIT_TASK_NAME(_Name)
	#define INIT_TASK_TASKNEXT
  #endif
  #if (OX_HANDLE_MTX)
	#define INIT_TASK_MYMUTEX			,(SEM_t *)NULL
  #else
	#define INIT_TASK_MYMUTEX
  #endif 
  #if (OX_USE_TASK_ARG)
	#define INIT_TASK_TASKARG			,(void *)NULL
  #else
	#define INIT_TASK_TASKARG
  #endif
  #if ((OX_SEARCH_ALGO) < 0)					/* When using link list instead of array			*/
	#define INIT_TASK_PRIONEXT			,(TSK_t *)NULL
	#define INIT_TASK_PRIOPREV			,(TSK_t *)NULL
  #else
	#define INIT_TASK_PRIONEXT
	#define INIT_TASK_PRIOPREV
  #endif
  #if (OX_STARVE_WAIT_MAX)
	#define INIT_TASK_STARVNEXT			,(TSK_t *)NULL
	#define INIT_TASK_STARVPREV			,(TSK_t *)NULL
	#define INIT_TASK_STARVLAST			,(TSK_t *)NULL
	#define INIT_TASK_STARVSTATE		,0
	#define INIT_TASK_STARVPRIO(_Prio)	,(_Prio)
	#define INIT_TASK_STARVTIMOUT		,0
	#if ((OX_STARVE_PRIO) < 0)
	  #define INIT_TASK_STARVPRIOMAX	,-(OX_STARVE_PRIO)
	#else
	  #define INIT_TASK_STARVPRIOMAX
	#endif
	#if ((OX_STARVE_RUN_MAX) < 0)
	  #define INIT_TASK_STARVRUNMAX		,-(OX_STARVE_RUN_MAX)
	#else
	  #define INIT_TASK_STARVRUNMAX
	#endif
	#if ((OX_STARVE_WAIT_MAX) < 0)
	  #define INIT_TASK_STARVWAITMAX	,-(OX_STARVE_WAIT_MAX)
	#else
	  #define INIT_TASK_STARVWAITMAX
	#endif
  #else
	#define INIT_TASK_STARVNEXT
	#define INIT_TASK_STARVPREV
	#define INIT_TASK_STARVLAST
	#define INIT_TASK_STARVSTATE
	#define INIT_TASK_STARVPRIO(_Prio)
	#define INIT_TASK_STARVTIMOUT
	#define INIT_TASK_STARVPRIOMAX
	#define INIT_TASK_STARVRUNMAX
	#define INIT_TASK_STARVWAITMAX
  #endif
  #if ((OX_TIMEOUT) > 0)
	#define INIT_TASK_TOUTNEXT			,(TSK_t *)NULL
	#define INIT_TASK_TOUTPREV			,(TSK_t *)NULL
	#define INIT_TASK_TICKTIME			,0
  #else
	#define INIT_TASK_TOUTNEXT
	#define INIT_TASK_TOUTPREV
	#define INIT_TASK_TICKTIME
  #endif
  #if ((OX_TASK_SUSPEND) || ((OX_TIMEOUT) > 0))
	#define INIT_TASK_PREVRETVAL		,0
  #else
	#define INIT_TASK_PREVRETVAL
  #endif
  #if (OX_TASK_SUSPEND)
    #define INIT_TASK_SUSPRQST			,0U
  #else
	#define INIT_TASK_SUSPRQST
  #endif
  #if (OX_ROUND_ROBIN)
	#define INIT_TASK_RRCNT				,0U
	#define INIT_TASK_RRTICK			,0U
	#if ((OX_ROUND_ROBIN) < 0)
		#define INIT_TASK_RRMAX			,(unsigned int)(OX_RR_TICK)
	#else
		#define INIT_TASK_RRMAX
	#endif
  #else
	#define INIT_TASK_RRCNT
	#define INIT_TASK_RRTICK
	#define INIT_TASK_RRMAX
  #endif
  #if (OX_COOPERATIVE)
	#define	INIT_TASK_YIELD				,0
  #else
	#define	INIT_TASK_YIELD
  #endif
  #if (OX_MTX_INVERSION)
	#define INIT_TASK_ORIPRIO(_Prio)	,(_Prio)
  #else
	#define INIT_TASK_ORIPRIO(_Prio)
  #endif
  #if (OX_EVENTS)
	#define INIT_TASK_EVENTRX			,0U
	#define INIT_TASK_EVENTAND			,0U
	#define INIT_TASK_EVENTOR			,0U
	#define INIT_TASK_EVENTACC			,0U
  #else
	#define INIT_TASK_EVENTRX
	#define INIT_TASK_EVENTAND
	#define INIT_TASK_EVENTOR
	#define INIT_TASK_EVENTACC
  #endif
  #if (OX_MAILBOX)
	#define INIT_TASK_MBOXMSG			,(intptr_t)0
  #else
	#define INIT_TASK_MBOXMSG
  #endif
  #if (OX_MEM_BLOCK)
	#define INIT_TASK_MBLKBUF			,(OSalign_t *)NULL
  #else
	#define INIT_TASK_MBLKBUF
  #endif
  #if (OX_STACK_CHECK)
	#define INIT_TASK_STACKLOW(x)		,(int32_t *)(void *)&(x)[0]
	#define INIT_TASK_STACKHIGH(x,y)	,((int32_t *)&(x))+(((y)/sizeof(int32_t))-1)
  #else
	#define INIT_TASK_STACKLOW(x)
	#define INIT_TASK_STACKHIGH(x,y)
  #endif

												/* We put stacksize in TSK_t.Priv.Value				*/
  #define TSK_STATIC(__Tsk, __Name, __Prio, __StackSize, __Fct)										\
   static OSalign_t __Tsk##__stack[((__StackSize)+sizeof(OSalign_t)-1)/sizeof(OSalign_t)];			\
   static _sTSK_t __Tsk##_s_ = {		\
   {									\
   (void *)&__Tsk##__stack[0]			\
   INIT_TASK_XTRADATA					\
   ,{									\
	 (TSK_t *)&__Tsk##_s_.Tsk			\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(&G_OSprivName[0])		\
	INIT_SEM_SEMANEXT					\
	,((int)sizeof(OSalign_t)*(((__StackSize)+(int)sizeof(OSalign_t)-1)/(int)sizeof(OSalign_t)))		\
	INIT_SEM_ISFCFS						\
	INIT_SEM_CEILPRIO					\
	INIT_SEM_PRIO_INV_ON				\
  }										\
  ,(SEM_t *)&__Tsk##_s_.Tsk.Priv		\
  ,(TSK_t *)NULL						\
  ,(TSK_t *)NULL						\
  INIT_TASK_LAST						\
  INIT_TASK_NAME(__Name)				\
  INIT_TASK_TASKNEXT					\
  INIT_TASK_MYMUTEX						\
  INIT_TASK_TASKARG						\
  INIT_TASK_PRIONEXT					\
  INIT_TASK_PRIOPREV					\
  INIT_TASK_STARVNEXT					\
  INIT_TASK_STARVPREV					\
  INIT_TASK_STARVLAST					\
  INIT_TASK_STARVSTATE					\
  INIT_TASK_STARVPRIO(__Prio)			\
  INIT_TASK_STARVTIMOUT					\
  INIT_TASK_STARVPRIOMAX				\
  INIT_TASK_STARVRUNMAX					\
  INIT_TASK_STARVWAITMAX				\
  INIT_TASK_TOUTNEXT					\
  INIT_TASK_TOUTPREV					\
  INIT_TASK_TICKTIME					\
  INIT_TASK_PREVRETVAL					\
  ,(__Prio)								\
  INIT_TASK_SUSPRQST					\
  INIT_TASK_RRCNT						\
  INIT_TASK_RRTICK						\
  INIT_TASK_RRMAX						\
  INIT_TASK_YIELD						\
  INIT_TASK_ORIPRIO(__Prio)				\
  INIT_TASK_EVENTACC					\
  INIT_TASK_EVENTRX						\
  INIT_TASK_EVENTAND					\
  INIT_TASK_EVENTOR						\
  INIT_TASK_MBOXMSG						\
  INIT_TASK_MBLKBUF						\
  INIT_TASK_STACKLOW(__Tsk##__stack)	\
  INIT_TASK_STACKHIGH((__Tsk##__stack), (__StackSize))	\
  },									\
  (__Fct)								\
 };										\
 TSK_t *(__Tsk) = &__Tsk##_s_.Tsk
#endif

#if (OX_STACK_CHECK)
  #define TSK_SC_SETUP(__Tsk)												\
	do {																	\
	  int      _ii;															\
	  int      _jj;															\
	  int32_t *Stack32;														\
		_jj     = __Tsk->StackHigh-__Tsk->StackLow;							\
		Stack32 = (int32_t *)__Tsk->StackLow;								\
		for (_ii=0 ; _ii<=_jj ; _ii++) {									\
			Stack32[_ii] = OX_STACK_MAGIC;									\
		}																	\
	} while(0)
#else
  #define TSK_SC_SETUP(__Tsk)			do{__Tsk=__Tsk;} while(0)
#endif

#ifdef OX_TASK_XTRA
  #define TSK_XTRA_SETUP(_Tsk_, _Fct_)	OX_TASK_XTRA(_Tsk_, _Fct_)
#else
  #define TSK_XTRA_SETUP(_Tsk_, _Fct_)	do{_Tsk_=_Tsk_;} while(0)
#endif

#ifndef __CC_ARM									/* Keil at -O2 & -O3 generates bad code			*/
  #define TSK_SETUP(_Tsk, _State)												\
	do {																		\
	  void (*Fct)(void);														\
		Fct = ((_sTSK_t *)(_Tsk))->TskFct;										\
		TSK_SC_SETUP(_Tsk);														\
		(_Tsk)->Stack      = OSstack((_Tsk)->Stack, (_Tsk)->Priv.Value, Fct);	\
		(_Tsk)->Priv.Value = 0;													\
		TSK_XTRA_SETUP(_Tsk, Fct);												\
		if ((_State) != 0) {													\
			(void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0);	\
		}																		\
	} while(0)
#else
  #define TSK_SETUP(_Tsk, _State)												\
	do {																		\
	  void (*Fct)(void);														\
		Fct = ((_sTSK_t *)(_Tsk))->TskFct;										\
		TSK_SC_SETUP(_Tsk);														\
		(_Tsk)->Stack	     = OSstack((_Tsk)->Stack, (_Tsk)->Priv.Value, Fct);	\
		(_Tsk)->Priv.Value   = ((int)(_Tsk)->Next);		/* repair */			\
		(_Tsk)->Priv.Blocked = (_Tsk);					/* repair */			\
		TSK_XTRA_SETUP(_Tsk, Fct);												\
		if ((_State) != 0) {													\
			(void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0);	\
		}																		\
	} while(0)
#endif

/* ------------------------------------------------	*/
/* Mailbox descriptor								*/

#if (OX_MAILBOX)

  typedef struct _MBX_t {
	SEM_t     MboxSema;							/* Semaphore when no buffer in the mailbox			*/
	intptr_t *Buffer;							/* Holds the queue / mailbox elements				*/
  #if (OX_NAMES)								/* Name of the queue /  mailbox						*/
	const char    *MbxName;
	struct _MBX_t *MbxNext;
  #endif
	int       Size;								/* Number of elements in the mailbox				*/
	int       RdIdx;							/* Read index										*/
	int       WrtIdx;							/* Write index										*/
  } MBX_t;

 #if ((OX_RUNTIME) >= 0)
  #if (OX_NAMES)
	#define INIT_MBX_NAME(_Name)		,(_Name)
	#define INIT_MBX_MBXNEXT			,(MBX_t *)NULL
  #else
	#define INIT_MBX_NAME(_Name)
	#define INIT_MBX_MBXNEXT
  #endif

  #define MBX_STATIC(__Var, __Name, __Size)		\
   static intptr_t __Var##__buf[(__Size)+1];	\
   MBX_t __Var##__ = {					\
   {									\
	 (TSK_t *)NULL						\
	,(TSK_t *)NULL						\
	INIT_SEM_MTXNEXT					\
	INIT_SEM_NAME(__Name)				\
	INIT_SEM_SEMANEXT					\
	,0									\
	INIT_SEM_ISFCFS						\
	INIT_SEM_CEILPRIO					\
	INIT_SEM_PRIO_INV_ON				\
   },									\
   &(__Var##__buf[0])					\
   INIT_MBX_NAME(__Name)				\
   INIT_MBX_MBXNEXT						\
   ,(__Size)							\
   ,0									\
   ,(__Size)							\
   };									\
   MBX_t *(__Var) = &(__Var##__)
 #endif

#endif

/* ------------------------------------------------	*/
/* Memory block descriptor							*/

#if (OX_MEM_BLOCK)

  typedef struct _MBLK_t {
	SEM_t      MblkSema;						/* Semaphore when no buffers are left in the pool	*/
	OSalign_t *Block;							/* Holds the head of the memory block linked list	*/
  #if (OX_NAMES)
	const char     *MblkName;					/* Name of the memory block							*/
	struct _MBLK_t *MblkNext;					/* Next memory block in the application				*/
  #endif
  } MBLK_t;

#endif

/* ------------------------------------------------	*/
/* Kernel requests bit masks (must fit in 16 bits)	*/
												/* MISRA has issues with (1U<<XX)					*/
#define ABASSI_BIN         (0x0001U)			/* Binary semaphore & extra tag to some commands	*/
#define ABASSI_WAIT        (0x0002U)			/* Regular semaphore waiting / mutex unblocking		*/
#define ABASSI_MTX         (0x0004U)			/* Mutex priority inheritance checking				*/
#define ABASSI_POST        (0x0008U)			/* Semaphore posting / mutex unblocking				*/
#define ABASSI_TOUT_RM     (0x0010U)			/* Remove from timeout linked list					*/
#define ABASSI_TOUT_ADD    (0x0020U)			/* Add in the timeout list							*/
#define ABASSI_TOUT_CHK    (0x0040U)			/* Check timer expiry's								*/
#define ABASSI_ADD_BLK     (0x0080U)			/* Add the task to the blocked list					*/
#define ABASSI_ADD_RUN     (0x0100U)			/* Add the task to the ready to run list			*/
#define ABASSI_NEW_PRIO    (0x0200U)			/* Set the task to a new priority					*/
#define ABASSI_EVENT       (0x0400U)			/* Event operation									*/
#define ABASSI_MBX_PUT     (0x0800U)			/* Put a mailbox message							*/
#define ABASSI_MBX_GET     (0x1000U)			/* Get a mailbox message							*/
#define ABASSI_RESUSP      (0x2000U)			/* Suspend or resume a task							*/
#define ABASSI_STARVE      (0x4000U)			/* Process starving (prio aging) protection			*/
#define ABASSI_TIM_SRV     (0x8000U)			/* Operation on timer services						*/

#define ABASSI_MBLK_GET	   (0x0180U)			/* Combines ABASSI_ADD_BLK & ABASSI_ADD_RUN			*/
#define ABASSI_MBLK_PUT	   (0x0181U)			/* This one + ABASSI_BIN							*/

#define ABASSI_TIM_RM      (ABASSI_BIN)			/* Added to timer service request to remove a timer	*/
#define ABASSI_TIM_ADD     (ABASSI_MTX)			/* Added to timer service request to add a timer	*/
#define ABASSI_TOUT_RM0    (ABASSI_BIN)			/* Remove from timeout list + zero TimedOut			*/
#define ABASSI_NEW_O_PRIO  (ABASSI_BIN)			/* New prio + set original							*/
#define ABASSI_SUSPEND     (ABASSI_BIN)			/* For ABASSI_RESUSP this means to suspend			*/

/* ------------------------------------------------	*/
/* Global variables									*/

extern volatile unsigned int G_OSstate[5];			/* If we are in an ISR or not and nesting level	*/
#if ((OX_N_INTERRUPTS) > 0)
  extern        void (*G_OSisrTbl[OX_N_INTERRUPTS])(void) REENT_POST;	/* Table used by ISRdispatch*/
#else
  extern        void (*G_OSisrTbl[1])(void) REENT_POST;	/* Imported/sized in ASM file, dummy size	*/
#endif
extern OSstrCnst_t     G_OSprivName[5];				/* Name for a task's private semaphore			*/
extern OSstrCnst_t     G_OSnoName[4];				/* Name for un-named components					*/
extern          TSK_t *G_OStaskNow;					/* Currently running task						*/
extern volatile int    G_OStimCnt;					/* RTOS time base counter tick					*/
extern          MTX_t *G_OSmutex;				    /* OS needs a mutex for some service creation	*/
													/* Made available even when OS_RUNTIME == 0		*/

/* ------------------------------------------------	*/
/* Atomic function prototypes						*/

extern int      Abassi      (unsigned int Ops, void *Ptr, intptr_t Arg) REENT_POST;
#if (OX_IDLE_STACK)
  extern TSK_t *TSKidle;
  extern void   IdleTask	(void) REENT_POST;
#endif
#if (OX_MAILBOX)
 #if (OX_RUNTIME)
  extern MBX_t *MBX__open   (const char *Name, int MbxSize, unsigned int IsFCFS) REENT_POST;
 #endif
  extern int MBXget			(MBX_t *Mbx,     intptr_t *Msg, int Tout) REENT_POST;
  extern int MBXput			(MBX_t *Mailbox, intptr_t  Msg, int Tout) REENT_POST;
#endif
#if ((OX_MEM_BLOCK) && (OX_RUNTIME))
  extern MBLK_t *MBLK__open	(const char *Name, int Nblock, int BlkSize, int MblkIsFCFS) REENT_POST;
  extern void   *MBLKalloc  (MBLK_t *MemBlk, int TimOut) REENT_POST;
#endif
extern void    *OSalloc     (size_t MemSize) REENT_POST;
extern void     OScontext   (TSK_t *NewTask) REENT_POST;
extern void     OSinvalidISR(void) REENT_POST;
extern int      OSisrInfo   (void) REENT_POST;
extern void    *OSstack     (void *StackBase, int StackSize, void (*TskFct)(void)) REENT_POST;
extern void     OSstart     (void);

extern  int			       OSintCtrl(int State);/* Function that deals with both dint() and eint()	*/
#define OSdint()		  (OSintCtrl(0))		/* Could split into 2 functions but only a few		*/
#define OSeint(x)	((void)OSintCtrl(x))		/* CPU cycles are added on OSdint(), not OSeint()	*/

#if (OX_RUNTIME)
  extern SEM_t *SEM__open   (const char *Name, unsigned int Type) REENT_POST;
  extern TSK_t *TSKcreate   (const char *Name, int TskPrio,int StackSize,void (*Fct)(void),int State)
						    REENT_POST;
#endif
extern void     TIMcallBack (void) REENT_POST;
#if ((OX_NAMES) && (OX_RUNTIME))
  extern TSK_t *TSKgetID	(const char *Name) REENT_POST;
#endif
extern void     TSKstart    (void (*TaskFct)(void)) REENT_POST;

/* ------------------------------------------------	*/
/* Function defines									*/

#if ((OX_N_INTERRUPTS) >= 0)
  #define OSisrInstall(Nmb, Fct)	(G_OSisrTbl[(Nmb)+(OX_ISR_TBL_OFFSET)]=(Fct))
#endif

typedef void TSKfct_t(void);						/* Useful for C++ when testing					*/

/* ------------------------------------------------	*/
/* Events related macros							*/

#if (OX_EVENTS)
  #define EVTget()					(G_OStaskNow->EventRX)
  #define EVTgetAcc()				(G_OStaskNow->EventAcc)
  #define EVTreset()				(G_OStaskNow->EventRX=0U)
  #define EVTresetAcc()				(G_OStaskNow->EventAcc=0U)
  #define EVTset(_Tsk, _Bits)		((void)Abassi((ABASSI_EVENT),(void *)(_Tsk),(intptr_t)(_Bits)))
  #define EVTwait(_And,_Or,_Tout)	(G_OStaskNow->Priv.Value = 0,									\
									 G_OStaskNow->EventOR    = (_Or),								\
									 G_OStaskNow->EventAND   = ((_And) == 0U)						\
									                         ? (_Or)								\
									                         : (_And),								\
									 Abassi(((ABASSI_WAIT)|(ABASSI_EVENT)), 						\
										    (void *)&(G_OStaskNow->Priv), (intptr_t)(_Tout)))
#endif


/* ------------------------------------------------	*/
/* Mailboxes related macros							*/

#if (OX_MAILBOX)
  #if (OX_RUNTIME)
	#define MBXopen(_Tsk, _Size)		(MBX__open((_Tsk), (_Size), 0U))
   #if (OX_FCFS)
	#define MBXopenFCFS(_Tsk, _Size)	(MBX__open((_Tsk), (_Size), 1U))
   #endif
  #endif

  #if (OX_FCFS)
	#define MBXsetFCFS(_Mbox)			((_Mbox)->MboxSema.IsFCFS=1)
	#define MBXnotFCFS(_Mbox)			((_Mbox)->MboxSema.IsFCFS=0)
  #endif
#endif

/* ------------------------------------------------	*/
/* Memory block related macros						*/

#if (OX_MEM_BLOCK)
	#define MBLKfree(__Mblk, _Buf)		(Abassi(ABASSI_MBLK_PUT, (__Mblk), (intptr_t)(_Buf)))

  #if (OX_RUNTIME)
	#define MBLKopen(_Tsk, _Nbk, _Sz)	(MBLK__open((_Tsk), (_Nbk), (_Sz), 0U))
   #if (OX_FCFS)
	#define MBLKopenFCFS(_Tsk,_Nbk,_Sz)	(MBLK__open((_Tsk), (_Nbk), (_Sz), 1U))
   #endif
  #endif

  #if (OX_FCFS)
	#define MBLKsetFCFS(_Mblk)			((_Mblk)->MblkSema.IsFCFS=1)
	#define MBLKnotFCFS(_Mblk)			((_Mblk)->MblkSema.IsFCFS=0)
  #endif

#endif

/* ------------------------------------------------	*/
/* Mutexes related macros							*/

#if (OX_RUNTIME)
  #define MTXopen(_Name)		(SEM__open((_Name), 1U))
#endif
#define MTXowner(_Mtx)			((_Mtx)->MtxOwner)
#define MTXlock(_Mtx, _Tout)	(Abassi(((ABASSI_WAIT)|(ABASSI_MTX)|(ABASSI_BIN)), (void *)(_Mtx),  \
								        (intptr_t)(_Tout)))
#define MTXunlock(_Mtx)			((void)Abassi(((ABASSI_POST)|(ABASSI_MTX)), (void *)(_Mtx),			\
								              (intptr_t)0))
#if (OX_FCFS)
 #if (OX_RUNTIME)
  #define MTXopenFCFS(_Name)	(SEM__open((_Name), 3U))
 #endif
  #define MTXsetFCFS(_Mtx)		((_Mtx)->IsFCFS=1)
  #define MTXnotFCFS(_Mtx)		((_Mtx)->IsFCFS=0)
#endif
#if ((OX_MTX_INVERSION) < 0)
  #define MTXgetCeilPrio(_Mtx)			(((volatile MTX_t *)(_Mtx))->CeilPrio)
  #define MTXsetCeilPrio(_Mtx, _Prio)	((_Mtx)->CeilPrio=(_Prio))
#endif
#if (OX_PRIO_INV_ON_OFF)
  #define MTXprioInvOff(_Mtx)	((_Mtx)->PrioInvOn=0)
  #define MTXprioInvOn(_Mtx)	((_Mtx)->PrioInvOn=1)
  #define MTXgetPrioInv(_Mtx)	((_Mtx)->PrioInvOn)
#endif

/* ------------------------------------------------	*/
/* Semaphores related macros						*/

#if (OX_RUNTIME)
  #define SEMopen(_Name)		(SEM__open((_Name), 0U))
#endif
#define SEMpost(_Sem)			((void)Abassi((ABASSI_POST), (void *)(_Sem), (intptr_t)0))
#define SEMwait(_Sem, _Tout)	(Abassi((ABASSI_WAIT), (void *)(_Sem), (intptr_t)(_Tout)))
#define SEMwaitBin(_Sem, _Tout)	(Abassi(((ABASSI_WAIT)|(ABASSI_BIN)), (void *)(_Sem),				\
								        (intptr_t)(_Tout)))
#if (OX_FCFS)
 #if (OX_RUNTIME)
  #define SEMopenFCFS(_Name)	(SEM__open((_Name), 2U))
 #endif
  #define SEMsetFCFS(_Sema)		((_Sema)->IsFCFS=1)
  #define SEMnotFCFS(_Sema)		((_Sema)->IsFCFS=0)
#endif


/* ------------------------------------------------	*/
/* Timer Services related macros					*/

#if (OX_TIMER_SRV)
  #if (OX_RUNTIME)
	extern  TIM_t *TIMopen(const char *Name) REENT_POST;
  #endif

	#define TIM_CALL_FCT		((0xFFFFU) & ~(ABASSI_TIM_SRV))
 	#define TIM_WRITE_DATA		((0xEEEEU) & ~(ABASSI_TIM_SRV))

	#define TIMarg(_Timer, _Arg)		((_Timer)->Arg   = (intptr_t)(_Arg))
	#define TIMtoAdd(_Timer, _ToAdd)	((_Timer)->ToAdd = (int)(_ToAdd))
	#define TIMkill(_Timer)				((void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_RM)),			\
										              (void *)(_Timer), (intptr_t)0))
	#define TIMperiod(_Timer, _Period)	((_Timer)->Period = (_Period))

	#define TIMdata(_Timer, _Addr, _Data, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (TIM_WRITE_DATA) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Addr);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period); 												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)),(void *)( _Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

  #if (OX_EVENTS)
	#define TIMevt(_Timer, _Tsk, _Data, _Expiry, _Period)											\
				do {																				\
					(_Timer)->Ops    = (ABASSI_EVENT) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Tsk);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = 0;															\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)
  #endif

	#define TIMfct(_Timer, _Fct, _Arg, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (TIM_CALL_FCT) | (ABASSI_TIM_SRV);							\
					(_Timer)->FctPtr = (_Fct);														\
					(_Timer)->Arg    = (intptr_t)(_Arg);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

  #if (OX_MAILBOX)
	#define TIMmbx(_Timer, _Mbx, _Data, _ToAdd, _Expiry, _Period)									\
				do {																				\
					(_Timer)->Ops    = (ABASSI_MBX_PUT) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Mbx);												\
					(_Timer)->Arg    = (intptr_t)(_Data);											\
					(_Timer)->ToAdd  = (int)(_ToAdd);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)
  #endif

	#define TIMmtx(_Timer, _Mtx, _Expiry, _Period)													\
				do {																				\
					(_Timer)->Ops    = (ABASSI_POST) | (ABASSI_MTX) | (ABASSI_TIM_SRV);				\
					(_Timer)->Ptr    = (void *)(_Mtx);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

	#define TIMsem(_Timer, _Sem, _Expiry, _Period)													\
				do {																				\
					(_Timer)->Ops    = (ABASSI_POST) | (ABASSI_TIM_SRV);							\
					(_Timer)->Ptr    = (void *)(_Sem);												\
					(_Timer)->Period = (int)(_Period);												\
					(void)Abassi(((ABASSI_TIM_SRV)|(ABASSI_TIM_ADD)), (void *)(_Timer),				\
					             (intptr_t)(_Expiry));												\
				} while(0)

#endif

/* ------------------------------------------------	*/
/* Tasks related macros								*/


#define TSKgetPrio(_Tsk)		(((volatile TSK_t *)(_Tsk))->Prio)
#define TSKmyID()				(G_OStaskNow)

#if (OX_PRIO_CHANGE)
  #define TSKsetPrio(_Tsk,_New) ((void)Abassi(((ABASSI_NEW_PRIO)|(ABASSI_NEW_O_PRIO)),				\
								              (void *)(_Tsk), (intptr_t)(_New)))
#endif


#if (OX_STARVE_WAIT_MAX)
  #if ((OX_STARVE_PRIO) < 0)
	#define TSKgetStrvPrio(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvPrioMax)
	#define TSKsetStrvPrio(_Tsk, _Prio)		((_Tsk)->StrvPrioMax = (_Prio))
  #else
	#define TSKgetStrvPrio(_Tsk)			(OX_STARVE_PRIO)
  #endif
  #if ((OX_STARVE_RUN_MAX) < 0)
	#define TSKgetStrvRunMax(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvRunMax)
	#define TSKsetStrvRunMax(_Tsk, _Cnt)	((_Tsk)->StrvRunMax  = (_Cnt))
  #else
	#define TSKgetStrvRunMax(_Tsk)			(OX_STARVE_RUN_MAX)
  #endif
  #if ((OX_STARVE_WAIT_MAX) < 0)
	#define TSKgetStrvWaitMax(_Tsk)			(((volatile TSK_t *)(_Tsk))->StrvWaitMax)
	#define TSKsetStrvWaitMax(_Tsk, _Cnt)	((_Tsk)->StrvWaitMax = (_Cnt))
  #else
	#define TSKgetStrvWaitMax(_Tsk)			(OX_STARVE_WAIT_MAX)
  #endif
#else
	#define TSKgetStrvPrio(_Tsk)			(OX_PRIO_MIN)
	#define TSKgetStrvRunMax(_Tsk)			0
	#define TSKgetStrvWaitMax(_Tsk)			0
#endif


#if (OX_TASK_SUSPEND)
  #define TSKresume(_Tsk)		((void)Abassi((ABASSI_RESUSP),  (void *)(_Tsk),  (intptr_t)0))
  #define TSKsuspend(_Tsk)		((void)Abassi(((ABASSI_RESUSP)|(ABASSI_SUSPEND)), (void *)(_Tsk),	\
								              (intptr_t)-1))
#else
  #define TSKresume(_Tsk)		((void)Abassi((ABASSI_POST), (void *)&((_Tsk)->Priv), (intptr_t)0))
#endif
#define TSKselfSusp()			((void)Abassi((ABASSI_WAIT),(void *)&(G_OStaskNow->Priv),(intptr_t)-1))

#if (OX_TIMER_US)
  extern void TIMtick(void) REENT_POST;
  #if ((OX_ROUND_ROBIN) < 0)
	#define TSKgetRR(_Tsk)		(((volatile TSK_t *)(_Tsk))->RRmax)
	#define TSKsetRR(_Tsk,_Tim)	((_Tsk)->RRmax = (_Tim))
  #else
   #ifdef OX_RR_TICK
	#define TSKgetRR(_Tsk)		(OX_RR_TICK)
   #endif
  #endif
  #if ((OX_TIMEOUT) > 0)
	#define TSKsleep(_TimeOut)	((void)Abassi((ABASSI_WAIT), (void *)&(G_OStaskNow->Priv),			\
								              (intptr_t)(_TimeOut)))
	#define TSKtimoutKill(_Tsk)	((void)Abassi((ABASSI_TOUT_RM), (void *)(_Tsk), (intptr_t)0))
  #endif
#else
	#define TSKgetRR(_Tsk)		(0U)
#endif

#if (OX_COOPERATIVE)								/* TSKyield() is !=  when cooperative or not	*/
	#define TSKyield()			do { 																\
									G_OStaskNow->Yield = 1;											\
									(void)Abassi(0U, NULL, (intptr_t)0);							\
								} while(0)
#else												/* Not coop, put the task at end of linked list	*/
	#define TSKyield()			do {																\
									if (G_OStaskNow->Next != (TSK_t *)NULL) {						\
										(void)Abassi(ABASSI_ADD_RUN, G_OStaskNow, (intptr_t)0);		\
									}																\
								} while(0)
#endif

#if (OX_EVENTS)
  #define OX_EVT_NOTBLK(_tsk)	(0U == ((volatile TSK_t *)(_tsk))->EventAND)
#else
  #define OX_EVT_NOTBLK(_tsk)	(1)
#endif
#if ((OX_TIMEOUT) > 0)
  #define OX_TOUT_NOTBLK(_tsk)	((TSK_t *)NULL == ((volatile TSK_t *)(_tsk))->ToutPrev)
#else
  #define OX_TOUT_NOTBLK(_tsk)	(1)
#endif

#define TSKstate(_Tsk)			((SEM_t *)NULL == (((volatile TSK_t *)(_Tsk))->Blocker)				\
								? 0								/* 0 is running		*/				\
								: (((((volatile TSK_t *)(_Tsk))->Blocker							\
								                           == (&((volatile TSK_t *)(_Tsk))->Priv))	\
								  && (OX_TOUT_NOTBLK(_Tsk))											\
								  && (OX_EVT_NOTBLK(_Tsk)))											\
								  ? -1			/* -ve is suspended, +ve is blocked */			 	\
								  :  1))

#define TSKisRdy(_Tsk)			((SEM_t *)NULL == ((volatile TSK_t *)(_Tsk))->Blocker)
#define TSKisBlk(_Tsk)			(TSKstate(_Tsk) > 0)
#define TSKisSusp(_Tsk)			((((volatile TSK_t *)(_Tsk))->Blocker								\
								                          == (&((volatile TSK_t *)(_Tsk))->Priv))	\
								    && (OX_TOUT_NOTBLK(_Tsk))										\
								    && (OX_EVT_NOTBLK(_Tsk)) )

#if (OX_USE_TASK_ARG)
  #define TSKsetArg(_Tsk, _Arg)	((_Tsk)->TaskArg=(_Arg))
  #define TSKgetArg()			(((volatile TSK_t *)(G_OStaskNow))->TaskArg)		
#endif

#if (OX_STACK_CHECK)
  #define OX_STACK_MAGIC		((int32_t)0xFAEEDA7AL)

  extern int TSKstkChk(TSK_t *Task, int IsFree) REENT_POST;

  #define TSKstkFree(_Tsk)		TSKstkChk((_Tsk), 1)
  #define TSKstkUsed(_Tsk)		TSKstkChk((_Tsk), 0)
#endif

/* ------------------------------------------------	*/
/* Logging related macros							*/


#if (OX_LOGGING_TYPE)
	#define LOG_ENABLE			0				/* Enable a specific event number					*/
	#define LOG_DISABLE			1				/* Enable a specific event number					*/
	#define LOG_ALL_ON			2				/* Enable the recording of all events				*/
	#define LOG_ALL_OFF			3				/* Disable the recording of all events				*/
	#define LOG_OFF				4				/* Turn off the recording (freeze it)				*/
	#define LOG_ON				5				/* Turn on the recording (Continue)					*/
	#define LOG_CONT			6				/* Start from scratch until OFF/buffer full			*/
	#define LOG_ONCE			7				/* Reset the recording: empty the buffer			*/
	#define LOG_DUMP_ALL		8				/* Dump all valid events from the buffer			*/
	#define LOG_DUMP_NEXT		9				/* Dump the next valid event from the buffer		*/
	#define LOG_GET_NEXT		10				/* Retrieve the next valid event from the buffer	*/

	#define LOGallOff()			((void)LOGctrl((LOG_ALL_OFF),   0U))
	#define LOGallOn()			((void)LOGctrl((LOG_ALL_ON),    0U))
	#define LOGdis(_Nmb)		((void)LOGctrl((LOG_DISABLE),  (unsigned int)(_Nmb)))
	#define LOGdumpAll()		((void)LOGctrl((LOG_DUMP_ALL),  0U))
	#define LOGdumpNext()		((void)LOGctrl((LOG_DUMP_NEXT), 0U))
	#define LOGenb(_Nmb)		((void)LOGctrl((LOG_ENABLE),   (unsigned int)(_Nmb)))
	#define LOGgetNext()		      (LOGctrl((LOG_GET_NEXT),  0U))
	#define LOGoff()			((void)LOGctrl((LOG_OFF),       0U))
	#define LOGon()				((void)LOGctrl((LOG_ON),        0U))
	#define LOGreset()			((void)LOGctrl((LOG_RESET),     0U))
	#define LOGstart()			((void)LOGctrl((LOG_START),     0U))

	extern const char   		*LOGctrl(int Ctrl, unsigned int Opt) REENT_POST;

#else
	#define LOGallOff()
	#define LOGallOn()
	#define LOGdis(_Nmb)
	#define LOGdumpAll()
	#define LOGdumpNext()
	#define LOGenb(_Nmb)
	#define LOGgetNext()
	#define LOGoff()
	#define LOGon()
	#define LOGreset()
	#define LOGstart()

#endif

#ifdef __cplusplus
  }
#endif

#endif											/* The whole file is conditional					*/

/* EOF */
